import asyncio
import json

import aiohttp
from aiohttp_retry import RetryClient, ExponentialRetry

from .bf2042 import bf_2042_gen_pic, bf2042_weapon
from .picture_builder import builder


data = {
    "userId": 1008269664784,
    "avatar": "",
    "userName": "Sansorano_Yume",
    "id": 1004152464784,
    "weapons": [
        {
            "type": "PDW",
            "weaponName": "PP-29",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_PP19-00f2ea5c.png",
            "id": "pri_pdw_pp29",
            "kills": 675,
            "damage": 79185,
            "bodyKills": 595,
            "headshotKills": 80,
            "hipfireKills": 140,
            "multiKills": 19,
            "accuracy": "11.2%",
            "killsPerMinute": 0.49,
            "damagePerMinute": 57.08,
            "headshots": "11.85%",
            "hitVKills": 5.27,
            "shotsHit": 3554,
            "shotsFired": 31733,
            "spawns": 202,
            "timeEquipped": 83240
        },
        {
            "type": "PDW",
            "weaponName": "MP9",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_MP9-6aa46b38.png",
            "id": "pri_pdw_mp9",
            "kills": 166,
            "damage": 23597,
            "bodyKills": 133,
            "headshotKills": 33,
            "hipfireKills": 40,
            "multiKills": 8,
            "accuracy": "8.81%",
            "killsPerMinute": 0.44,
            "damagePerMinute": 63.18,
            "headshots": "19.88%",
            "hitVKills": 7.25,
            "shotsHit": 1204,
            "shotsFired": 13667,
            "spawns": 247,
            "timeEquipped": 22410
        },
        {
            "type": "PDW",
            "weaponName": "K30",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_kriss-41643e7c.png",
            "id": "pri_pdw_k30",
            "kills": 257,
            "damage": 29549,
            "bodyKills": 211,
            "headshotKills": 46,
            "hipfireKills": 73,
            "multiKills": 13,
            "accuracy": "13.89%",
            "killsPerMinute": 1.11,
            "damagePerMinute": 127.32,
            "headshots": "17.9%",
            "hitVKills": 7.23,
            "shotsHit": 1858,
            "shotsFired": 13381,
            "spawns": 58,
            "timeEquipped": 13925
        },
        {
            "type": "PDW",
            "weaponName": "PBX-45",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SMG45-e28be5b5.png",
            "id": "pri_pdw_pbx45",
            "kills": 276,
            "damage": 34088,
            "bodyKills": 240,
            "headshotKills": 36,
            "hipfireKills": 60,
            "multiKills": 3,
            "accuracy": "14.3%",
            "killsPerMinute": 0.81,
            "damagePerMinute": 99.43,
            "headshots": "13.04%",
            "hitVKills": 6.44,
            "shotsHit": 1778,
            "shotsFired": 12433,
            "spawns": 2,
            "timeEquipped": 20570
        },
        {
            "type": "Bolt Action",
            "weaponName": "SWS-10",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_TRG_M10-ebbdfe15.png",
            "id": "pri_ba_sws10",
            "kills": 18,
            "damage": 2943,
            "bodyKills": 9,
            "headshotKills": 9,
            "hipfireKills": 6,
            "multiKills": 0,
            "accuracy": "14.68%",
            "killsPerMinute": 0.22,
            "damagePerMinute": 35.74,
            "headshots": "50.0%",
            "hitVKills": 2.67,
            "shotsHit": 48,
            "shotsFired": 327,
            "spawns": 25,
            "timeEquipped": 4940
        },
        {
            "type": "Bolt Action",
            "weaponName": "NTW-50",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_NTW-20-76c5dc49.png",
            "id": "pri_ba_ntw50",
            "kills": 114514,
            "damage": 14956,
            "bodyKills": 113,
            "headshotKills": 18,
            "hipfireKills": 32,
            "multiKills": 0,
            "accuracy": "18.44%",
            "killsPerMinute": 0.37,
            "damagePerMinute": 41.92,
            "headshots": "13.74%",
            "hitVKills": 1.26,
            "shotsHit": 165,
            "shotsFired": 895,
            "spawns": 59,
            "timeEquipped": 21405
        },
        {
            "type": "Bolt Action",
            "weaponName": "DXR-1",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_DSR1-c3413554.png",
            "id": "pri_ba_dxr1",
            "kills": 231,
            "damage": 35753,
            "bodyKills": 71,
            "headshotKills": 160,
            "hipfireKills": 57,
            "multiKills": 2,
            "accuracy": "23.63%",
            "killsPerMinute": 0.37,
            "damagePerMinute": 57.58,
            "headshots": "69.26%",
            "hitVKills": 2.41,
            "shotsHit": 556,
            "shotsFired": 2353,
            "spawns": 173,
            "timeEquipped": 37255
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AM40",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_AM17-32eab750.png",
            "id": "pri_ar_am40",
            "kills": 545,
            "damage": 63621,
            "bodyKills": 413,
            "headshotKills": 132,
            "hipfireKills": 90,
            "multiKills": 7,
            "accuracy": "13.39%",
            "killsPerMinute": 1.05,
            "damagePerMinute": 122.29,
            "headshots": "24.22%",
            "hitVKills": 5.47,
            "shotsHit": 2979,
            "shotsFired": 22245,
            "spawns": 2,
            "timeEquipped": 31215
        },
        {
            "type": "DMR",
            "weaponName": "VCAR",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_keltec-c9ee5698.png",
            "id": "pri_dmr_vcar",
            "kills": 6,
            "damage": 931,
            "bodyKills": 6,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "12.09%",
            "killsPerMinute": 0.91,
            "damagePerMinute": 141.42,
            "headshots": "0.0%",
            "hitVKills": 5.5,
            "shotsHit": 33,
            "shotsFired": 273,
            "spawns": 0,
            "timeEquipped": 395
        },
        {
            "type": "Crossbows",
            "weaponName": "Ghostmaker R10",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Cobra-6ca88503.png",
            "id": "pri_misc_cb_ghstmkr",
            "kills": 47,
            "damage": 5097,
            "bodyKills": 43,
            "headshotKills": 4,
            "hipfireKills": 30,
            "multiKills": 0,
            "accuracy": "33.62%",
            "killsPerMinute": 0.46,
            "damagePerMinute": 49.57,
            "headshots": "8.51%",
            "hitVKills": 2.51,
            "shotsHit": 118,
            "shotsFired": 351,
            "spawns": 0,
            "timeEquipped": 6170
        },
        {
            "type": "DMR",
            "weaponName": "BSV-M",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_VSS-d4610752.png",
            "id": "pri_dmr_vss",
            "kills": 697,
            "damage": 84778,
            "bodyKills": 563,
            "headshotKills": 134,
            "hipfireKills": 75,
            "multiKills": 1,
            "accuracy": "15.57%",
            "killsPerMinute": 0.9,
            "damagePerMinute": 109.58,
            "headshots": "19.23%",
            "hitVKills": 4.94,
            "shotsHit": 3443,
            "shotsFired": 22106,
            "spawns": 0,
            "timeEquipped": 46420
        },
        {
            "type": "Sidearm",
            "weaponName": "G57",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_glock-237f6b21.png",
            "id": "sa_g57",
            "kills": 219,
            "damage": 21071,
            "bodyKills": 185,
            "headshotKills": 34,
            "hipfireKills": 113,
            "multiKills": 5,
            "accuracy": "18.36%",
            "killsPerMinute": 1.85,
            "damagePerMinute": 177.69,
            "headshots": "15.53%",
            "hitVKills": 3.96,
            "shotsHit": 868,
            "shotsFired": 4728,
            "spawns": 458,
            "timeEquipped": 7115
        },
        {
            "type": "Sidearm",
            "weaponName": "M44",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Taurus-4ab7a57b.png",
            "id": "sa_m44",
            "kills": 6,
            "damage": 690,
            "bodyKills": 4,
            "headshotKills": 2,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "16.67%",
            "killsPerMinute": 0.25,
            "damagePerMinute": 28.26,
            "headshots": "33.33%",
            "hitVKills": 2.17,
            "shotsHit": 13,
            "shotsFired": 78,
            "spawns": 470,
            "timeEquipped": 1465
        },
        {
            "type": "Shotguns",
            "weaponName": "MCS-880",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_remington-a054d3fb.png",
            "id": "pri_misc_sg_mcs880",
            "kills": 166,
            "damage": 18982,
            "bodyKills": 143,
            "headshotKills": 23,
            "hipfireKills": 94,
            "multiKills": 7,
            "accuracy": "72.72%",
            "killsPerMinute": 1.15,
            "damagePerMinute": 131.82,
            "headshots": "13.86%",
            "hitVKills": 3.98,
            "shotsHit": 661,
            "shotsFired": 909,
            "spawns": 17,
            "timeEquipped": 8640
        },
        {
            "type": "Shotguns",
            "weaponName": "12M Auto",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_saiga-aa338425.png",
            "id": "pri_misc_sg_12mauto",
            "kills": 311,
            "damage": 33664,
            "bodyKills": 289,
            "headshotKills": 22,
            "hipfireKills": 304,
            "multiKills": 15,
            "accuracy": "40.37%",
            "killsPerMinute": 1.18,
            "damagePerMinute": 127.19,
            "headshots": "7.07%",
            "hitVKills": 6.81,
            "shotsHit": 2118,
            "shotsFired": 5246,
            "spawns": 2,
            "timeEquipped": 15880
        },
        {
            "type": "Sidearm",
            "weaponName": "MP28",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_MP17-dce205e4.png",
            "id": "sa_mp28",
            "kills": 7,
            "damage": 443,
            "bodyKills": 6,
            "headshotKills": 1,
            "hipfireKills": 1,
            "multiKills": 1,
            "accuracy": "10.61%",
            "killsPerMinute": 0.87,
            "damagePerMinute": 54.8,
            "headshots": "14.29%",
            "hitVKills": 2.71,
            "shotsHit": 19,
            "shotsFired": 179,
            "spawns": 196,
            "timeEquipped": 485
        },
        {
            "type": "Railguns",
            "weaponName": "Rorsch Mk-4",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_RailgunRifle-be9d51b7.png",
            "id": "pri_misc_rg_mk4",
            "kills": 811,
            "damage": 104600,
            "bodyKills": 256,
            "headshotKills": 555,
            "hipfireKills": 6,
            "multiKills": 22,
            "accuracy": "17.53%",
            "killsPerMinute": 0.85,
            "damagePerMinute": 109.24,
            "headshots": "68.43%",
            "hitVKills": 2.41,
            "shotsHit": 1955,
            "shotsFired": 11154,
            "spawns": 0,
            "timeEquipped": 57450
        },
        {
            "type": "LMG",
            "weaponName": "Avancys",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Avancys-5cf323f7.png",
            "id": "pri_lmg_avancys",
            "kills": 612,
            "damage": 74389,
            "bodyKills": 475,
            "headshotKills": 137,
            "hipfireKills": 74,
            "multiKills": 25,
            "accuracy": "8.41%",
            "killsPerMinute": 1.02,
            "damagePerMinute": 124.47,
            "headshots": "22.39%",
            "hitVKills": 5.82,
            "shotsHit": 3561,
            "shotsFired": 42352,
            "spawns": 0,
            "timeEquipped": 35860
        },
        {
            "type": "LMG",
            "weaponName": "PKP-BP",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Pecheng-69395d9b.png",
            "id": "pri_lmg_pkpbp",
            "kills": 199,
            "damage": 24720,
            "bodyKills": 157,
            "headshotKills": 42,
            "hipfireKills": 23,
            "multiKills": 41,
            "accuracy": "7.2%",
            "killsPerMinute": 0.77,
            "damagePerMinute": 96.12,
            "headshots": "21.11%",
            "hitVKills": 6.08,
            "shotsHit": 1209,
            "shotsFired": 16796,
            "spawns": 84,
            "timeEquipped": 15430
        },
        {
            "type": "LMG",
            "weaponName": "LCMG",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_LAMG-74966966.png",
            "id": "pri_lmg_lcmg",
            "kills": 227,
            "damage": 28242,
            "bodyKills": 176,
            "headshotKills": 51,
            "hipfireKills": 38,
            "multiKills": 4,
            "accuracy": "9.52%",
            "killsPerMinute": 1.05,
            "damagePerMinute": 131.26,
            "headshots": "22.47%",
            "hitVKills": 5.93,
            "shotsHit": 1347,
            "shotsFired": 14150,
            "spawns": 15,
            "timeEquipped": 12910
        },
        {
            "type": "Bolt Action",
            "weaponName": "M24 Sniper",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M24Sniper-d2b3578f.png",
            "id": "pri_ba_m24",
            "kills": 1,
            "damage": 156,
            "bodyKills": 0,
            "headshotKills": 1,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "28.57%",
            "killsPerMinute": 0.17,
            "damagePerMinute": 26.0,
            "headshots": "100.0%",
            "hitVKills": 2.0,
            "shotsHit": 2,
            "shotsFired": 7,
            "spawns": 2,
            "timeEquipped": 360
        },
        {
            "type": "Assault Rifles",
            "weaponName": "M16A3",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M16A3-13468163.png",
            "id": "pri_ar_m16a3",
            "kills": 137,
            "damage": 17118,
            "bodyKills": 110,
            "headshotKills": 27,
            "hipfireKills": 28,
            "multiKills": 1,
            "accuracy": "11.15%",
            "killsPerMinute": 0.78,
            "damagePerMinute": 97.91,
            "headshots": "19.71%",
            "hitVKills": 5.51,
            "shotsHit": 755,
            "shotsFired": 6774,
            "spawns": 0,
            "timeEquipped": 10490
        },
        {
            "type": "Lever-Action Carbines",
            "weaponName": "GVT 45-70",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_marlin-2b091f63.png",
            "id": "pri_misc_lac_gvt4570",
            "kills": 8,
            "damage": 1010,
            "bodyKills": 5,
            "headshotKills": 3,
            "hipfireKills": 2,
            "multiKills": 0,
            "accuracy": "36.96%",
            "killsPerMinute": 1.09,
            "damagePerMinute": 137.73,
            "headshots": "37.5%",
            "hitVKills": 2.12,
            "shotsHit": 17,
            "shotsFired": 46,
            "spawns": 3,
            "timeEquipped": 440
        },
        {
            "type": "PDW",
            "weaponName": "P90",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_P90-833ca028.png",
            "id": "pri_pdw_p90",
            "kills": 101,
            "damage": 9988,
            "bodyKills": 80,
            "headshotKills": 21,
            "hipfireKills": 12,
            "multiKills": 7,
            "accuracy": "21.08%",
            "killsPerMinute": 5.41,
            "damagePerMinute": 535.07,
            "headshots": "20.79%",
            "hitVKills": 5.15,
            "shotsHit": 520,
            "shotsFired": 2467,
            "spawns": 0,
            "timeEquipped": 1120
        },
        {
            "type": "Sidearm",
            "weaponName": "PF51",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_PF51-569f3708.png",
            "id": "sa_pf51",
            "kills": 32,
            "damage": 3342,
            "bodyKills": 28,
            "headshotKills": 4,
            "hipfireKills": 18,
            "multiKills": 0,
            "accuracy": "16.03%",
            "killsPerMinute": 1.44,
            "damagePerMinute": 150.77,
            "headshots": "12.5%",
            "hitVKills": 7.84,
            "shotsHit": 251,
            "shotsFired": 1566,
            "spawns": 0,
            "timeEquipped": 1330
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AC-42",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_NBR-04c26c82.png",
            "id": "pri_ar_ac42",
            "kills": 260,
            "damage": 34317,
            "bodyKills": 214,
            "headshotKills": 46,
            "hipfireKills": 37,
            "multiKills": 2,
            "accuracy": "13.04%",
            "killsPerMinute": 0.48,
            "damagePerMinute": 63.29,
            "headshots": "17.69%",
            "hitVKills": 5.47,
            "shotsHit": 1423,
            "shotsFired": 10910,
            "spawns": 41,
            "timeEquipped": 32535
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AK-24",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_AK12-bc93fb15.png",
            "id": "pri_ar_ak24",
            "kills": 740,
            "damage": 91296,
            "bodyKills": 612,
            "headshotKills": 128,
            "hipfireKills": 174,
            "multiKills": 8,
            "accuracy": "16.65%",
            "killsPerMinute": 0.65,
            "damagePerMinute": 79.87,
            "headshots": "17.3%",
            "hitVKills": 6.6,
            "shotsHit": 4886,
            "shotsFired": 29354,
            "spawns": 3,
            "timeEquipped": 68580
        },
        {
            "type": "Assault Rifles",
            "weaponName": "M5A3",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SLX-71737176.png",
            "id": "pri_ar_m5a3",
            "kills": 548,
            "damage": 70661,
            "bodyKills": 432,
            "headshotKills": 116,
            "hipfireKills": 103,
            "multiKills": 4,
            "accuracy": "12.35%",
            "killsPerMinute": 0.59,
            "damagePerMinute": 76.6,
            "headshots": "21.17%",
            "hitVKills": 6.55,
            "shotsHit": 3587,
            "shotsFired": 29033,
            "spawns": 81,
            "timeEquipped": 55350
        },
        {
            "type": "Assault Rifles",
            "weaponName": "SFAR-M GL",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SCARMK17-fbe5ac70.png",
            "id": "pri_ar_sfarm",
            "kills": 471,
            "damage": 59430,
            "bodyKills": 373,
            "headshotKills": 98,
            "hipfireKills": 108,
            "multiKills": 9,
            "accuracy": "15.37%",
            "killsPerMinute": 0.83,
            "damagePerMinute": 105.25,
            "headshots": "20.81%",
            "hitVKills": 5.81,
            "shotsHit": 2736,
            "shotsFired": 17804,
            "spawns": 55,
            "timeEquipped": 33880
        },
        {
            "type": "Bolt Action",
            "weaponName": "K98 Sniper",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_Kar98K-Sniper-78649ce0.png",
            "id": "pri_ba_k98s",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "DMR",
            "weaponName": "DM7",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_DDM4-90304a0b.png",
            "id": "pri_dmr_dm7",
            "kills": 71,
            "damage": 9230,
            "bodyKills": 45,
            "headshotKills": 26,
            "hipfireKills": 2,
            "multiKills": 1,
            "accuracy": "18.88%",
            "killsPerMinute": 0.46,
            "damagePerMinute": 60.33,
            "headshots": "36.62%",
            "hitVKills": 3.93,
            "shotsHit": 279,
            "shotsFired": 1478,
            "spawns": 9,
            "timeEquipped": 9180
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AKS-74u",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_AKS74u-839ebbac.png",
            "id": "pri_misc_carb_aks",
            "kills": 203,
            "damage": 22739,
            "bodyKills": 172,
            "headshotKills": 31,
            "hipfireKills": 18,
            "multiKills": 7,
            "accuracy": "23.51%",
            "killsPerMinute": 4.8,
            "damagePerMinute": 537.14,
            "headshots": "15.27%",
            "hitVKills": 5.48,
            "shotsHit": 1113,
            "shotsFired": 4734,
            "spawns": 0,
            "timeEquipped": 2540
        },
        {
            "type": "LMG",
            "weaponName": "M240B",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M240B-436660f8.png",
            "id": "pri_lmg_m240b",
            "kills": 11,
            "damage": 1201,
            "bodyKills": 10,
            "headshotKills": 1,
            "hipfireKills": 2,
            "multiKills": 0,
            "accuracy": "1.83%",
            "killsPerMinute": 0.16,
            "damagePerMinute": 17.88,
            "headshots": "9.09%",
            "hitVKills": 4.18,
            "shotsHit": 46,
            "shotsFired": 2516,
            "spawns": 0,
            "timeEquipped": 4030
        },
        {
            "type": "Sidearm",
            "weaponName": "MP412 REX",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_MP-412REX-bdaa3806.png",
            "id": "sa_rex",
            "kills": 7,
            "damage": 780,
            "bodyKills": 5,
            "headshotKills": 2,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "12.59%",
            "killsPerMinute": 1.22,
            "damagePerMinute": 135.65,
            "headshots": "28.57%",
            "hitVKills": 2.57,
            "shotsHit": 18,
            "shotsFired": 143,
            "spawns": 7,
            "timeEquipped": 345
        },
        {
            "type": "Sidearm",
            "weaponName": "MP443",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_MP443-5cb9a041.png",
            "id": "sa_mp443",
            "kills": 1,
            "damage": 130,
            "bodyKills": 1,
            "headshotKills": 0,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "20.83%",
            "killsPerMinute": 0.63,
            "damagePerMinute": 82.11,
            "headshots": "0.0%",
            "hitVKills": 5.0,
            "shotsHit": 5,
            "shotsFired": 24,
            "spawns": 3,
            "timeEquipped": 95
        },
        {
            "type": "Sidearm",
            "weaponName": "M93R",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M93R-dbc63f6c.png",
            "id": "sa_m93r",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Shotguns",
            "weaponName": "SPAS-12",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SPAS12-d9391052.png",
            "id": "pri_misc_sg_spas12",
            "kills": 2,
            "damage": 232,
            "bodyKills": 2,
            "headshotKills": 0,
            "hipfireKills": 2,
            "multiKills": 0,
            "accuracy": "100.0%",
            "killsPerMinute": 1.26,
            "damagePerMinute": 146.53,
            "headshots": "0.0%",
            "hitVKills": 5.5,
            "shotsHit": 11,
            "shotsFired": 11,
            "spawns": 0,
            "timeEquipped": 95
        },
        {
            "type": "LMG",
            "weaponName": "M60E4",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M60E4-63fe87c7.png",
            "id": "pri_lmg_m60e4",
            "kills": 2,
            "damage": 192,
            "bodyKills": 1,
            "headshotKills": 1,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "17.07%",
            "killsPerMinute": 3.0,
            "damagePerMinute": 288.0,
            "headshots": "50.0%",
            "hitVKills": 3.5,
            "shotsHit": 7,
            "shotsFired": 41,
            "spawns": 0,
            "timeEquipped": 40
        },
        {
            "type": "DMR",
            "weaponName": "M39 EMR",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M39EMR-5abaffd4.png",
            "id": "pri_dmr_m39",
            "kills": 76,
            "damage": 8597,
            "bodyKills": 67,
            "headshotKills": 9,
            "hipfireKills": 10,
            "multiKills": 1,
            "accuracy": "15.24%",
            "killsPerMinute": 0.81,
            "damagePerMinute": 91.78,
            "headshots": "11.84%",
            "hitVKills": 2.61,
            "shotsHit": 198,
            "shotsFired": 1299,
            "spawns": 0,
            "timeEquipped": 5620
        },
        {
            "type": "Carbines",
            "weaponName": "A-91",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_A91-884f4c20.png",
            "id": "pri_misc_carb_a91",
            "kills": 4,
            "damage": 432,
            "bodyKills": 4,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "14.67%",
            "killsPerMinute": 1.78,
            "damagePerMinute": 192.0,
            "headshots": "0.0%",
            "hitVKills": 5.5,
            "shotsHit": 22,
            "shotsFired": 150,
            "spawns": 0,
            "timeEquipped": 135
        },
        {
            "type": "DMR",
            "weaponName": "SVD",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_SVD-5eb4de27.png",
            "id": "pri_dmr_svd",
            "kills": 2,
            "damage": 292,
            "bodyKills": 1,
            "headshotKills": 1,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "24.0%",
            "killsPerMinute": 0.3,
            "damagePerMinute": 44.35,
            "headshots": "50.0%",
            "hitVKills": 3.0,
            "shotsHit": 6,
            "shotsFired": 25,
            "spawns": 0,
            "timeEquipped": 395
        },
        {
            "type": "Carbines",
            "weaponName": "ACW-R",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_ACW-R-d7872974.png",
            "id": "pri_misc_carb_acwr",
            "kills": 19,
            "damage": 2282,
            "bodyKills": 15,
            "headshotKills": 4,
            "hipfireKills": 9,
            "multiKills": 0,
            "accuracy": "12.06%",
            "killsPerMinute": 1.09,
            "damagePerMinute": 130.4,
            "headshots": "21.05%",
            "hitVKills": 4.89,
            "shotsHit": 93,
            "shotsFired": 771,
            "spawns": 0,
            "timeEquipped": 1050
        },
        {
            "type": "Bolt Action",
            "weaponName": "M98B",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M98B-7afe78dc.png",
            "id": "pri_ba_m98b",
            "kills": 1,
            "damage": 237,
            "bodyKills": 0,
            "headshotKills": 1,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "36.36%",
            "killsPerMinute": 0.36,
            "damagePerMinute": 86.18,
            "headshots": "100.0%",
            "hitVKills": 4.0,
            "shotsHit": 4,
            "shotsFired": 11,
            "spawns": 0,
            "timeEquipped": 165
        },
        {
            "type": "Assault Rifles",
            "weaponName": "M416",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_M416-8f4cd9da.png",
            "id": "pri_ar_m416",
            "kills": 87,
            "damage": 10284,
            "bodyKills": 71,
            "headshotKills": 16,
            "hipfireKills": 16,
            "multiKills": 0,
            "accuracy": "14.45%",
            "killsPerMinute": 0.91,
            "damagePerMinute": 108.16,
            "headshots": "18.39%",
            "hitVKills": 5.77,
            "shotsHit": 502,
            "shotsFired": 3473,
            "spawns": 0,
            "timeEquipped": 5705
        },
        {
            "type": "Assault Rifles",
            "weaponName": "SCAR-H",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_SCAR-H-a1658fa6.png",
            "id": "pri_misc_carb_scarh",
            "kills": 2,
            "damage": 440,
            "bodyKills": 2,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "19.09%",
            "killsPerMinute": 1.14,
            "damagePerMinute": 251.43,
            "headshots": "0.0%",
            "hitVKills": 10.5,
            "shotsHit": 21,
            "shotsFired": 110,
            "spawns": 0,
            "timeEquipped": 105
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AEK-971",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_AEK-971_BF3-db5476e3.png",
            "id": "pri_ar_aek",
            "kills": 65,
            "damage": 6996,
            "bodyKills": 53,
            "headshotKills": 12,
            "hipfireKills": 13,
            "multiKills": 2,
            "accuracy": "16.22%",
            "killsPerMinute": 0.48,
            "damagePerMinute": 51.69,
            "headshots": "18.46%",
            "hitVKills": 5.46,
            "shotsHit": 355,
            "shotsFired": 2188,
            "spawns": 0,
            "timeEquipped": 8120
        },
        {
            "type": "",
            "weaponName": "NVK-P125",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_BullpupPistol-2371a03c.png",
            "id": "sa_nvkp125",
            "kills": 83,
            "damage": 8831,
            "bodyKills": 66,
            "headshotKills": 17,
            "hipfireKills": 32,
            "multiKills": 9,
            "accuracy": "21.74%",
            "killsPerMinute": 2.6,
            "damagePerMinute": 276.69,
            "headshots": "20.48%",
            "hitVKills": 3.94,
            "shotsHit": 327,
            "shotsFired": 1504,
            "spawns": 3,
            "timeEquipped": 1915
        },
        {
            "type": "Shotguns",
            "weaponName": "NVK-S22",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_CB12-e0c8db2a.png",
            "id": "pri_misc_sg_nvks22",
            "kills": 53,
            "damage": 6074,
            "bodyKills": 49,
            "headshotKills": 4,
            "hipfireKills": 45,
            "multiKills": 0,
            "accuracy": "38.65%",
            "killsPerMinute": 1.15,
            "damagePerMinute": 132.28,
            "headshots": "7.55%",
            "hitVKills": 4.75,
            "shotsHit": 252,
            "shotsFired": 652,
            "spawns": 3,
            "timeEquipped": 2755
        },
        {
            "type": "DMR",
            "weaponName": "SVK",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_chukavin-1f41118c.png",
            "id": "pri_dmr_svk",
            "kills": 87,
            "damage": 14230,
            "bodyKills": 64,
            "headshotKills": 23,
            "hipfireKills": 3,
            "multiKills": 0,
            "accuracy": "15.73%",
            "killsPerMinute": 0.35,
            "damagePerMinute": 57.0,
            "headshots": "26.44%",
            "hitVKills": 4.17,
            "shotsHit": 363,
            "shotsFired": 2308,
            "spawns": 59,
            "timeEquipped": 14980
        },
        {
            "type": "PDW",
            "weaponName": "XM8 Compact",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_XM8-C-48f3a3fd.png",
            "id": "pri_pdw_xm8c",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 1,
            "timeEquipped": 0
        },
        {
            "type": "Assault Rifle",
            "weaponName": "StG 44",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_STG44-050b204d.png",
            "id": "pri_ar_stg",
            "kills": 8,
            "damage": 1006,
            "bodyKills": 6,
            "headshotKills": 2,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "10.42%",
            "killsPerMinute": 0.87,
            "damagePerMinute": 109.75,
            "headshots": "25.0%",
            "hitVKills": 5.88,
            "shotsHit": 47,
            "shotsFired": 451,
            "spawns": 0,
            "timeEquipped": 550
        },
        {
            "type": "Assault Rifle",
            "weaponName": "M2 Carbine",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_M2Carbine-59b11087.png",
            "id": "pri_ar_m2carbine",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "PDW",
            "weaponName": "PP-2000",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_PP2000-4cc58b87.png",
            "id": "pri_pdw_pp2000",
            "kills": 512,
            "damage": 52128,
            "bodyKills": 446,
            "headshotKills": 66,
            "hipfireKills": 73,
            "multiKills": 53,
            "accuracy": "21.25%",
            "killsPerMinute": 4.33,
            "damagePerMinute": 440.83,
            "headshots": "12.89%",
            "hitVKills": 4.55,
            "shotsHit": 2331,
            "shotsFired": 10970,
            "spawns": 0,
            "timeEquipped": 7095
        },
        {
            "type": "Sidearm",
            "weaponName": "M1911",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M1911-e4ea2148.png",
            "id": "sa_m1911",
            "kills": 6,
            "damage": 403,
            "bodyKills": 5,
            "headshotKills": 1,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "8.5%",
            "killsPerMinute": 0.48,
            "damagePerMinute": 32.46,
            "headshots": "16.67%",
            "hitVKills": 2.83,
            "shotsHit": 17,
            "shotsFired": 200,
            "spawns": 3,
            "timeEquipped": 745
        },
        {
            "type": "Shotguns",
            "weaponName": "DAO-12",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_DAO-12-eb920cae.png",
            "id": "pri_misc_sg_dao12",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "LMG",
            "weaponName": "Type 88",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_Type-88-f0f49ebd.png",
            "id": "pri_lmg_t88",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 7,
            "spawns": 0,
            "timeEquipped": 5
        },
        {
            "type": "LMG",
            "weaponName": "M249 SAW",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_M249_SAW_BF3-c07c0a9b.png",
            "id": "pri_lmg_m249",
            "kills": 2,
            "damage": 349,
            "bodyKills": 2,
            "headshotKills": 0,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "11.72%",
            "killsPerMinute": 1.41,
            "damagePerMinute": 246.35,
            "headshots": "0.0%",
            "hitVKills": 7.5,
            "shotsHit": 15,
            "shotsFired": 128,
            "spawns": 0,
            "timeEquipped": 85
        },
        {
            "type": "LMG",
            "weaponName": "XM8 LMG",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_XM8-LMG-ba39a043.png",
            "id": "pri_lmg_xm8lmg",
            "kills": 153,
            "damage": 17476,
            "bodyKills": 121,
            "headshotKills": 32,
            "hipfireKills": 22,
            "multiKills": 1,
            "accuracy": "11.74%",
            "killsPerMinute": 1.4,
            "damagePerMinute": 160.33,
            "headshots": "20.92%",
            "hitVKills": 5.5,
            "shotsHit": 841,
            "shotsFired": 7162,
            "spawns": 0,
            "timeEquipped": 6540
        },
        {
            "type": "Bolt Action",
            "weaponName": "GOL Sniper Magnum",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_GOL-35dfe25f.png",
            "id": "pri_ba_gol",
            "kills": 153,
            "damage": 21735,
            "bodyKills": 85,
            "headshotKills": 68,
            "hipfireKills": 45,
            "multiKills": 3,
            "accuracy": "27.85%",
            "killsPerMinute": 0.63,
            "damagePerMinute": 89.05,
            "headshots": "44.44%",
            "hitVKills": 2.25,
            "shotsHit": 344,
            "shotsFired": 1235,
            "spawns": 0,
            "timeEquipped": 14645
        },
        {
            "type": "LMG",
            "weaponName": "M60",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M60-d9864992.png",
            "id": "pri_lmg_m60",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "DMR",
            "weaponName": "M1 Garand",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M1Garand-fe29246e.png",
            "id": "pri_dmr_garand",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Bolt Action",
            "weaponName": "M95 Sniper",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_M95-092cdbf6.png",
            "id": "pri_ba_m95",
            "kills": 0,
            "damage": 81,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "5.56%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 17.05,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 1,
            "shotsFired": 18,
            "spawns": 0,
            "timeEquipped": 285
        },
        {
            "type": "Assault Rifles",
            "weaponName": "XM8 Prototype",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_XM8-0ed8134e.png",
            "id": "pri_ar_xm8",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "SMG-PDW",
            "weaponName": "MP 40",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_MP40-458414da.png",
            "id": "pri_pdw_mp40",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Sidearm",
            "weaponName": "Walther P38",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_WaltherP38-28dd9754.png",
            "id": "sa_p38",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "LMG",
            "weaponName": "BAR 1918",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_BAR1918_A2-38a25aed.png",
            "id": "pri_lmg_bar",
            "kills": 1,
            "damage": 168,
            "bodyKills": 1,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "16.67%",
            "killsPerMinute": 0.57,
            "damagePerMinute": 96.0,
            "headshots": "0.0%",
            "hitVKills": 5.0,
            "shotsHit": 5,
            "shotsFired": 30,
            "spawns": 0,
            "timeEquipped": 105
        },
        {
            "type": "LMG",
            "weaponName": "Bren LMG",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Bren-2f6ebe1f.png",
            "id": "pri_lmg_bren",
            "kills": 5,
            "damage": 319,
            "bodyKills": 3,
            "headshotKills": 2,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "4.78%",
            "killsPerMinute": 0.62,
            "damagePerMinute": 39.88,
            "headshots": "40.0%",
            "hitVKills": 3.0,
            "shotsHit": 15,
            "shotsFired": 314,
            "spawns": 0,
            "timeEquipped": 480
        },
        {
            "type": "Assault Rifles",
            "weaponName": "AN94",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_AN-94-9b8bea79.png",
            "id": "pri_ar_an94",
            "kills": 5,
            "damage": 499,
            "bodyKills": 4,
            "headshotKills": 1,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "14.02%",
            "killsPerMinute": 1.82,
            "damagePerMinute": 181.45,
            "headshots": "20.0%",
            "hitVKills": 4.6,
            "shotsHit": 23,
            "shotsFired": 164,
            "spawns": 0,
            "timeEquipped": 165
        },
        {
            "type": "Bolt Action",
            "weaponName": "K98",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_Kar98K-dde47354.png",
            "id": "pri_ba_k98",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Assault Rifles",
            "weaponName": "G3A3",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_G3A3-3969e77d.png",
            "id": "pri_ar_g3a3",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "",
            "weaponName": "AS Val",
            "image": "https://cdn.gametools.network/weapons/bf2042/ALX__weapon_default_AS-Val-86b653d8.png",
            "id": "pri_pdw_asval",
            "kills": 0,
            "damage": 77,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "8.11%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 57.75,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 3,
            "shotsFired": 37,
            "spawns": 0,
            "timeEquipped": 80
        },
        {
            "type": "DMR",
            "weaponName": "M1 Garand Sniper",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M1Garand-fe29246e.png",
            "id": "pri_dmr_garandsniper",
            "kills": 0,
            "damage": 273,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "15.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 148.91,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 6,
            "shotsFired": 40,
            "spawns": 0,
            "timeEquipped": 110
        },
        {
            "type": "Assault Rifles",
            "weaponName": "M16A2",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M16A2-a87f25c7.png",
            "id": "pri_ar_m16a2",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Bolt Action",
            "weaponName": "No 4",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_LeeEnfield-No4-57cac005.png",
            "id": "pri_ba_no4engineer",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Bolt Action",
            "weaponName": "No 4 Sniper",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_Sniper-No4-e2d380d5.png",
            "id": "pri_ba_no4",
            "kills": 3,
            "damage": 489,
            "bodyKills": 1,
            "headshotKills": 2,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "25.0%",
            "killsPerMinute": 0.59,
            "damagePerMinute": 96.2,
            "headshots": "66.67%",
            "hitVKills": 2.33,
            "shotsHit": 7,
            "shotsFired": 28,
            "spawns": 0,
            "timeEquipped": 305
        },
        {
            "type": "DMR",
            "weaponName": "Gewerhr 43 ZF4",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Gewehr43-ZF4-aeb3e6b8.png",
            "id": "pri_dmr_gewehrzf4",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "DMR",
            "weaponName": "Gewerhr 43",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Gewehr43-37bf9ea4.png",
            "id": "pri_dmr_gewehr",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "SMG-PDW",
            "weaponName": "Thompson",
            "image": "https://cdn.gametools.network/weapons/bf2042/RUM__weapon_default_M1A1Thompson-a625fdf8.png",
            "id": "pri_pdw_tommy",
            "kills": 1,
            "damage": 252,
            "bodyKills": 1,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "6.83%",
            "killsPerMinute": 0.43,
            "damagePerMinute": 108.0,
            "headshots": "0.0%",
            "hitVKills": 11.0,
            "shotsHit": 11,
            "shotsFired": 161,
            "spawns": 0,
            "timeEquipped": 140
        },
        {
            "type": "LMG",
            "weaponName": "RPT-3",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_RPT-31-d4ec514a.png",
            "id": "pri_lmg_rpt31",
            "kills": 256,
            "damage": 34208,
            "bodyKills": 197,
            "headshotKills": 59,
            "hipfireKills": 41,
            "multiKills": 10,
            "accuracy": "8.27%",
            "killsPerMinute": 0.51,
            "damagePerMinute": 68.56,
            "headshots": "23.05%",
            "hitVKills": 6.79,
            "shotsHit": 1738,
            "shotsFired": 21008,
            "spawns": 2,
            "timeEquipped": 29935
        },
        {
            "type": "SMG-PDW",
            "weaponName": "AC9",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SMG45-e28be5b5.png",
            "id": "pri_misc_carb_ac9",
            "kills": 496,
            "damage": 52875,
            "bodyKills": 432,
            "headshotKills": 64,
            "hipfireKills": 186,
            "multiKills": 43,
            "accuracy": "20.52%",
            "killsPerMinute": 0.59,
            "damagePerMinute": 63.25,
            "headshots": "12.9%",
            "hitVKills": 5.77,
            "shotsHit": 2861,
            "shotsFired": 13941,
            "spawns": 0,
            "timeEquipped": 50155
        },
        {
            "type": "Shotguns",
            "weaponName": "Super 500",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_SuperShorty-54bfa64f.png",
            "id": "sa_super500",
            "kills": 22,
            "damage": 2363,
            "bodyKills": 20,
            "headshotKills": 2,
            "hipfireKills": 18,
            "multiKills": 2,
            "accuracy": "80.8%",
            "killsPerMinute": 1.34,
            "damagePerMinute": 143.94,
            "headshots": "9.09%",
            "hitVKills": 4.59,
            "shotsHit": 101,
            "shotsFired": 125,
            "spawns": 2,
            "timeEquipped": 985
        },
        {
            "type": "Assault Rifles",
            "weaponName": "RM68",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_LAMG-74966966.png",
            "id": "pri_ar_rm68",
            "kills": 912,
            "damage": 111921,
            "bodyKills": 695,
            "headshotKills": 217,
            "hipfireKills": 105,
            "multiKills": 9,
            "accuracy": "13.24%",
            "killsPerMinute": 0.82,
            "damagePerMinute": 100.91,
            "headshots": "23.79%",
            "hitVKills": 5.1,
            "shotsHit": 4647,
            "shotsFired": 35111,
            "spawns": 0,
            "timeEquipped": 66545
        },
        {
            "type": "Assault Rifles",
            "weaponName": "MTAR-21",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_MTAR21-2e42c468.png",
            "id": "pri_pdw_mtar",
            "kills": 10,
            "damage": 1457,
            "bodyKills": 8,
            "headshotKills": 2,
            "hipfireKills": 2,
            "multiKills": 0,
            "accuracy": "13.97%",
            "killsPerMinute": 1.46,
            "damagePerMinute": 213.22,
            "headshots": "20.0%",
            "hitVKills": 6.9,
            "shotsHit": 69,
            "shotsFired": 494,
            "spawns": 0,
            "timeEquipped": 410
        },
        {
            "type": "Sidearm",
            "weaponName": "Mk VI Revolver",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_MkVI_Revolver-9f5b0bfe.png",
            "id": "sa_mkvi",
            "kills": 1,
            "damage": 18,
            "bodyKills": 1,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "6.25%",
            "killsPerMinute": 1.71,
            "damagePerMinute": 30.86,
            "headshots": "0.0%",
            "hitVKills": 1.0,
            "shotsHit": 1,
            "shotsFired": 16,
            "spawns": 0,
            "timeEquipped": 35
        },
        {
            "type": "LMG",
            "weaponName": "RPK-74M",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_RPK-74M-d52bf4cd.png",
            "id": "pri_lmg_rpk74m",
            "kills": 9,
            "damage": 900,
            "bodyKills": 8,
            "headshotKills": 1,
            "hipfireKills": 1,
            "multiKills": 0,
            "accuracy": "14.29%",
            "killsPerMinute": 1.32,
            "damagePerMinute": 131.71,
            "headshots": "11.11%",
            "hitVKills": 5.44,
            "shotsHit": 49,
            "shotsFired": 343,
            "spawns": 0,
            "timeEquipped": 410
        },
        {
            "type": "Bolt Action",
            "weaponName": "XCE BAR",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_TTS_Xceed-b66895b3.png",
            "id": "pri_ba_xceed",
            "kills": 34,
            "damage": 6717,
            "bodyKills": 8,
            "headshotKills": 26,
            "hipfireKills": 16,
            "multiKills": 0,
            "accuracy": "19.61%",
            "killsPerMinute": 0.31,
            "damagePerMinute": 61.34,
            "headshots": "76.47%",
            "hitVKills": 3.24,
            "shotsHit": 110,
            "shotsFired": 561,
            "spawns": 0,
            "timeEquipped": 6570
        },
        {
            "type": "Assault Rifles",
            "weaponName": "GEW-46",
            "image": "https://cdn.gametools.network/weapons/bf2042/gadget_default_G36C-9adbbaef.png",
            "id": "pri_ar_g36c",
            "kills": 377,
            "damage": 45904,
            "bodyKills": 298,
            "headshotKills": 79,
            "hipfireKills": 64,
            "multiKills": 7,
            "accuracy": "13.35%",
            "killsPerMinute": 0.76,
            "damagePerMinute": 93.05,
            "headshots": "20.95%",
            "hitVKills": 5.49,
            "shotsHit": 2070,
            "shotsFired": 15507,
            "spawns": 0,
            "timeEquipped": 29600
        },
        {
            "type": "Sidearm",
            "weaponName": "BFP.50",
            "image": "https://cdn.gametools.network/weapons/bf2042/gadget_default_DesertEagle-5bb177e7.png",
            "id": "sa_eagle",
            "kills": 168,
            "damage": 17830,
            "bodyKills": 124,
            "headshotKills": 44,
            "hipfireKills": 26,
            "multiKills": 11,
            "accuracy": "24.0%",
            "killsPerMinute": 1.75,
            "damagePerMinute": 186.21,
            "headshots": "26.19%",
            "hitVKills": 1.97,
            "shotsHit": 331,
            "shotsFired": 1379,
            "spawns": 3,
            "timeEquipped": 5745
        },
        {
            "type": "Shotguns",
            "weaponName": "12G Automatic",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_BrowningA5-9ef7afe3.png",
            "id": "pri_misc_sg_12gauto",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "SMG-PDW",
            "weaponName": "Sten",
            "image": "https://cdn.gametools.network/weapons/bf2042/weapon_default_Sten-e97e81c6.png",
            "id": "pri_pdw_sten",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "",
            "weaponName": "Melee",
            "image": "",
            "id": "melee",
            "kills": 151,
            "damage": 12749,
            "bodyKills": 150,
            "headshotKills": 1,
            "hipfireKills": 151,
            "multiKills": 7,
            "accuracy": "54.59%",
            "killsPerMinute": 3.53,
            "damagePerMinute": 297.64,
            "headshots": "0.66%",
            "hitVKills": 1.58,
            "shotsHit": 238,
            "shotsFired": 436,
            "spawns": 0,
            "timeEquipped": 2570
        },
        {
            "type": "Assault Rifles",
            "weaponName": "G3",
            "image": "",
            "id": "pri_ar_g3",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Launcher",
            "weaponName": "Panzerschreck",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__weapon_default_Panzerschrek-932d81bb.png",
            "id": "",
            "kills": 0,
            "damage": 0,
            "bodyKills": 0,
            "headshotKills": 0,
            "hipfireKills": 0,
            "multiKills": 0,
            "accuracy": "0.0%",
            "killsPerMinute": 0.0,
            "damagePerMinute": 0.0,
            "headshots": "0.0%",
            "hitVKills": 0.0,
            "shotsHit": 0,
            "shotsFired": 0,
            "spawns": 0,
            "timeEquipped": 0
        },
        {
            "type": "Assault Rifles",
            "weaponName": "VHX D3",
            "image": "https://cdn.gametools.network/weapons/bf2042/wep_itemReveal_VHSD2-bdd794b7.png",
            "id": "pri_ar_vhxd3",
            "kills": 406,
            "damage": 49398,
            "bodyKills": 316,
            "headshotKills": 90,
            "hipfireKills": 73,
            "multiKills": 6,
            "accuracy": "14.77%",
            "killsPerMinute": 1.38,
            "damagePerMinute": 167.88,
            "headshots": "22.17%",
            "hitVKills": 5.81,
            "shotsHit": 2360,
            "shotsFired": 15974,
            "spawns": 0,
            "timeEquipped": 17655
        }
    ],
    "vehicles": [
        {
            "type": "Helicopter",
            "vehicleName": "MD540 Nightbird",
            "image": "https://cdn.gametools.network/vehicles/bf2042/LittleBird-54f140e1.png",
            "id": "air_heli_nightbird",
            "kills": 72,
            "killsPerMinute": 0.39,
            "damage": 13767,
            "spawns": 70,
            "roadKills": 1,
            "passengerAssists": 81,
            "multiKills": 21,
            "distanceTraveled": 288053,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 13,
            "assists": 81,
            "callIns": 38,
            "damageTo": 5737,
            "destroyed": 32,
            "timeIn": 11060
        },
        {
            "type": "Plane",
            "vehicleName": "F-35E Panther",
            "image": "https://cdn.gametools.network/vehicles/bf2042/F35-6e81cc1d.png",
            "id": "air_plane_f35e",
            "kills": 114514,
            "killsPerMinute": 0.24,
            "damage": 9625,
            "spawns": 73,
            "roadKills": 2,
            "passengerAssists": 0,
            "multiKills": 4,
            "distanceTraveled": 702254,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 30,
            "assists": 0,
            "callIns": 50,
            "damageTo": 4827,
            "destroyed": 31,
            "timeIn": 9420
        },
        {
            "type": "Land",
            "vehicleName": "M5C Bolte",
            "image": "https://cdn.gametools.network/vehicles/bf2042/FNSSZAHA-673ae567.png",
            "id": "sur_land_bolte",
            "kills": 233,
            "killsPerMinute": 0.7,
            "damage": 56114,
            "spawns": 100,
            "roadKills": 47,
            "passengerAssists": 18,
            "multiKills": 43,
            "distanceTraveled": 144509,
            "driverAssists": 72,
            "vehiclesDestroyedWith": 16,
            "assists": 90,
            "callIns": 83,
            "damageTo": 3033,
            "destroyed": 20,
            "timeIn": 20035
        },
        {
            "type": "Helicopter",
            "vehicleName": "AH-64GX Apache Warchief",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Apache-6fd7d17c.png",
            "id": "air_heli_apache",
            "kills": 52,
            "killsPerMinute": 0.97,
            "damage": 7691,
            "spawns": 27,
            "roadKills": 0,
            "passengerAssists": 11,
            "multiKills": 14,
            "distanceTraveled": 94016,
            "driverAssists": 7,
            "vehiclesDestroyedWith": 12,
            "assists": 18,
            "callIns": 14,
            "damageTo": 2789,
            "destroyed": 30,
            "timeIn": 3215
        },
        {
            "type": "Helicopter",
            "vehicleName": "MV-38 Condor",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Osprey-dc88f56b.png",
            "id": "air_heli_condor",
            "kills": 29,
            "killsPerMinute": 0.27,
            "damage": 4792,
            "spawns": 66,
            "roadKills": 0,
            "passengerAssists": 45,
            "multiKills": 6,
            "distanceTraveled": 175254,
            "driverAssists": 35,
            "vehiclesDestroyedWith": 1,
            "assists": 80,
            "callIns": 41,
            "damageTo": 4634,
            "destroyed": 28,
            "timeIn": 6455
        },
        {
            "type": "Land",
            "vehicleName": "EBAA Wildcat",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Jaguar-00976ef9.png",
            "id": "sur_land_wildcat",
            "kills": 218,
            "killsPerMinute": 0.27,
            "damage": 44545,
            "spawns": 71,
            "roadKills": 4,
            "passengerAssists": 156,
            "multiKills": 38,
            "distanceTraveled": 90893,
            "driverAssists": 148,
            "vehiclesDestroyedWith": 69,
            "assists": 304,
            "callIns": 37,
            "damageTo": 9444,
            "destroyed": 61,
            "timeIn": 49215
        },
        {
            "type": "Land",
            "vehicleName": "M1A5",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Abrams-666376b1.png",
            "id": "sur_land_m1a5",
            "kills": 276,
            "killsPerMinute": 1.03,
            "damage": 38495,
            "spawns": 39,
            "roadKills": 0,
            "passengerAssists": 265,
            "multiKills": 66,
            "distanceTraveled": 71679,
            "driverAssists": 133,
            "vehiclesDestroyedWith": 32,
            "assists": 398,
            "callIns": 29,
            "damageTo": 8406,
            "destroyed": 50,
            "timeIn": 16035
        },
        {
            "type": "Helicopter",
            "vehicleName": "RAH-68 Huron",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Shoshone-770d49a8.png",
            "id": "air_heli_huron",
            "kills": 47,
            "killsPerMinute": 0.62,
            "damage": 9043,
            "spawns": 30,
            "roadKills": 0,
            "passengerAssists": 28,
            "multiKills": 11,
            "distanceTraveled": 157946,
            "driverAssists": 13,
            "vehiclesDestroyedWith": 7,
            "assists": 41,
            "callIns": 18,
            "damageTo": 4186,
            "destroyed": 32,
            "timeIn": 4580
        },
        {
            "type": "Land",
            "vehicleName": "LATV4 Recon",
            "image": "https://cdn.gametools.network/vehicles/bf2042/HDTStorm-508855e2.png",
            "id": "sur_land_latv4",
            "kills": 138,
            "killsPerMinute": 0.08,
            "damage": 24578,
            "spawns": 99,
            "roadKills": 7,
            "passengerAssists": 9,
            "multiKills": 29,
            "distanceTraveled": 90359,
            "driverAssists": 22,
            "vehiclesDestroyedWith": 7,
            "assists": 31,
            "callIns": 82,
            "damageTo": 8154,
            "destroyed": 60,
            "timeIn": 100525
        },
        {
            "type": "Helicopter",
            "vehicleName": "AH-6J Little Bird",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__AH-6_LittleBird-020fc28b.png",
            "id": "air_heli_ah6j",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 44,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Amphibious",
            "vehicleName": "LCAA Hovercraft",
            "image": "https://cdn.gametools.network/vehicles/bf2042/HoverCraft-3b2863af.png",
            "id": "sur_amph_lcaa",
            "kills": 19,
            "killsPerMinute": 0.34,
            "damage": 5210,
            "spawns": 21,
            "roadKills": 3,
            "passengerAssists": 4,
            "multiKills": 2,
            "distanceTraveled": 23791,
            "driverAssists": 21,
            "vehiclesDestroyedWith": 1,
            "assists": 25,
            "callIns": 21,
            "damageTo": 2132,
            "destroyed": 17,
            "timeIn": 3335
        },
        {
            "type": "Land",
            "vehicleName": "EBLC-Ram",
            "image": "https://cdn.gametools.network/vehicles/bf2042/CallIn_PanhardCrab-defd9d4c.png",
            "id": "sur_land_crab",
            "kills": 293,
            "killsPerMinute": 0.89,
            "damage": 39061,
            "spawns": 55,
            "roadKills": 11,
            "passengerAssists": 297,
            "multiKills": 49,
            "distanceTraveled": 129529,
            "driverAssists": 120,
            "vehiclesDestroyedWith": 27,
            "assists": 417,
            "callIns": 24,
            "damageTo": 3593,
            "destroyed": 18,
            "timeIn": 19765
        },
        {
            "type": "Amphibious",
            "vehicleName": "MAV",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Ripsaw-5d8b6a01.png",
            "id": "sur_amph_mav",
            "kills": 224,
            "killsPerMinute": 0.92,
            "damage": 34183,
            "spawns": 46,
            "roadKills": 13,
            "passengerAssists": 312,
            "multiKills": 84,
            "distanceTraveled": 73976,
            "driverAssists": 272,
            "vehiclesDestroyedWith": 9,
            "assists": 584,
            "callIns": 36,
            "damageTo": 5050,
            "destroyed": 31,
            "timeIn": 14550
        },
        {
            "type": "Land",
            "vehicleName": "EMKV90-TOR",
            "image": "https://cdn.gametools.network/vehicles/bf2042/VEH_Icon_CV90_Tank-f5cd269c.png",
            "id": "sur_land_emkv90",
            "kills": 219,
            "killsPerMinute": 0.74,
            "damage": 38118,
            "spawns": 32,
            "roadKills": 0,
            "passengerAssists": 13,
            "multiKills": 64,
            "distanceTraveled": 32219,
            "driverAssists": 39,
            "vehiclesDestroyedWith": 107,
            "assists": 52,
            "callIns": 33,
            "damageTo": 4708,
            "destroyed": 25,
            "timeIn": 17670
        },
        {
            "type": "Helicopter",
            "vehicleName": "UH-60",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__UH-60-85e1dc03.png",
            "id": "air_heli_uh60",
            "kills": 2,
            "killsPerMinute": 1.09,
            "damage": 204,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 1,
            "distanceTraveled": 2888,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 1,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 110
        },
        {
            "type": "Land",
            "vehicleName": "LAV-AD",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__LAV-AD-81ad7f0b.png",
            "id": "sur_amph_lavad",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "M1A2",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__M1A2_Abrams-c5c29b15.png",
            "id": "sur_land_m1a2",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 94,
            "spawns": 1,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 585,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 1,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 165
        },
        {
            "type": "Land",
            "vehicleName": "M1114",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__HMMVW-3f72cce0.png",
            "id": "sur_land_m1114",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "M1161 ITV",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__Growler_ITV-495b7d78.png",
            "id": "sur_land_m1161itv",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "LAV-25",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__LAV-25-5e26ccd6.png",
            "id": "sur_amph_lav25",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Helicopter",
            "vehicleName": "AH-64 Apache",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__AH64E_Apache-95ea5891.png",
            "id": "air_heli_ah64",
            "kills": 8,
            "killsPerMinute": 1.45,
            "damage": 1389,
            "spawns": 3,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 2,
            "distanceTraveled": 11848,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 6,
            "assists": 0,
            "callIns": 3,
            "damageTo": 140,
            "destroyed": 1,
            "timeIn": 330
        },
        {
            "type": "Land",
            "vehicleName": "AAV-7A1 AMTRAC",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__AAV-7A1_AMTRAC-b0d97585.png",
            "id": "sur_amph_amtrac",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 173,
            "destroyed": 1,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "M3A3 Bradley",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__M3A3-Bradley-c62fcd8a.png",
            "id": "sur_amph_m3a3",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 52,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 5,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 100,
            "destroyed": 1,
            "timeIn": 10
        },
        {
            "type": "Land",
            "vehicleName": "M4 Sherman",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__M4-Sherman-eadb90c6.png",
            "id": "sur_land_m4sherm",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "Willys MB",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__WillysJeep-f813f4c0.png",
            "id": "sur_land_willys",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "M10 Wolverine",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__M10Wolverine-617e2e5f.png",
            "id": "sur_land_m10w",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Plane",
            "vehicleName": "Spitfire",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__SpitfirePlane_MX-Va-45ed6b74.png",
            "id": "air_plane_spitfire",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Plane",
            "vehicleName": "B-17 Bomber",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__B-17-Bomber-a32375cc.png",
            "id": "air_plane_b17",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 205,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 5
        },
        {
            "type": "Land",
            "vehicleName": "M3 Halftrack",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__M3-e8eabeb4.png",
            "id": "sur_land_m3ht",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 45,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 15
        },
        {
            "type": "Plane",
            "vehicleName": "F/A-18",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__FA-18_Hornet-fbaeedb9.png",
            "id": "air_plane_fa18",
            "kills": 4,
            "killsPerMinute": 0.39,
            "damage": 673,
            "spawns": 3,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 2,
            "distanceTraveled": 56741,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 2,
            "assists": 0,
            "callIns": 3,
            "damageTo": 181,
            "destroyed": 0,
            "timeIn": 615
        },
        {
            "type": "Plane",
            "vehicleName": "A-10 Warthog",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__A-10_Thunderbolt-II-8fd06332.png",
            "id": "air_plane_a10",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 168,
            "destroyed": 2,
            "timeIn": 0
        },
        {
            "type": "Plane",
            "vehicleName": "SU-57 FELON",
            "image": "https://cdn.gametools.network/vehicles/bf2042/SU57-8c465ace.png",
            "id": "air_plane_su57",
            "kills": 1919,
            "killsPerMinute": 0.25,
            "damage": 5173,
            "spawns": 51,
            "roadKills": 2,
            "passengerAssists": 0,
            "multiKills": 5,
            "distanceTraveled": 446771,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 13,
            "assists": 0,
            "callIns": 30,
            "damageTo": 4569,
            "destroyed": 24,
            "timeIn": 3600000
        },
        {
            "type": "Helicopter",
            "vehicleName": "Mi-240 Super Hind",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Mi24-f2f92880.png",
            "id": "air_heli_mi240",
            "kills": 222,
            "killsPerMinute": 0.18,
            "damage": 38642,
            "spawns": 93,
            "roadKills": 2,
            "passengerAssists": 248,
            "multiKills": 64,
            "distanceTraveled": 2852821,
            "driverAssists": 327,
            "vehiclesDestroyedWith": 8,
            "assists": 575,
            "callIns": 72,
            "damageTo": 5656,
            "destroyed": 28,
            "timeIn": 72885
        },
        {
            "type": "Helicopter",
            "vehicleName": "YG-99 Hannibal",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Hannibal-3e287b57.png",
            "id": "air_heli_hannibal",
            "kills": 22,
            "killsPerMinute": 0.44,
            "damage": 3991,
            "spawns": 26,
            "roadKills": 0,
            "passengerAssists": 14,
            "multiKills": 0,
            "distanceTraveled": 98642,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 3,
            "assists": 14,
            "callIns": 16,
            "damageTo": 2454,
            "destroyed": 16,
            "timeIn": 3030
        },
        {
            "type": "Plane",
            "vehicleName": "Ju-87 Stuka",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__Stuka_JU-78_B-1-356f2845.png",
            "id": "air_plane_stuka",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Plane",
            "vehicleName": "Bf 109",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__BF109-388ac401.png",
            "id": "air_plane_bf109",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Amphibious",
            "vehicleName": "BMP-2",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__BMP-2-8ce5888b.png",
            "id": "sur_amph_bmp2",
            "kills": 9,
            "killsPerMinute": 10.8,
            "damage": 889,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 3,
            "multiKills": 5,
            "distanceTraveled": 85,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 3,
            "callIns": 0,
            "damageTo": 228,
            "destroyed": 6,
            "timeIn": 50
        },
        {
            "type": "Plane",
            "vehicleName": "Su-35BM",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__SU-35BM-275a430d.png",
            "id": "air_plane_su35bm",
            "kills": 1,
            "killsPerMinute": 0.24,
            "damage": 229,
            "spawns": 6,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 21497,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 1,
            "assists": 0,
            "callIns": 6,
            "damageTo": 30,
            "destroyed": 1,
            "timeIn": 245
        },
        {
            "type": "Land",
            "vehicleName": "Tiger I",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__tiger-74a6f11e.png",
            "id": "sur_land_tigeri",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 18,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "VDV Buggy",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__VDV_Buggy-24a34e8f.png",
            "id": "sur_land_vdv",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 408,
            "destroyed": 8,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "Sd. Kfz 251 Halftrack",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__HanomagHalftrack-27a8128a.png",
            "id": "sur_land_kfzht",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 1,
            "multiKills": 0,
            "distanceTraveled": 14,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 1,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 5
        },
        {
            "type": "Land",
            "vehicleName": "Kubelwagen",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__KubelwagenTransport-536c4c14.png",
            "id": "sur_land_kubel",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Land",
            "vehicleName": "9K22 Tunguska-M",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__9K22_Tunguska-26412514.png",
            "id": "sur_land_tunguska",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 5,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 23,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 44,
            "destroyed": 0,
            "timeIn": 10
        },
        {
            "type": "Land",
            "vehicleName": "T-90",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__T90-8a6f6d04.png",
            "id": "sur_land_t90",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 75,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Amphibious",
            "vehicleName": "3937 Vodnik",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__GAZ-3937_Vodnik-70c94108.png",
            "id": "sur_amph_vodnik",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 16,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 169,
            "destroyed": 3,
            "timeIn": 10
        },
        {
            "type": "Helicopter",
            "vehicleName": "Z-11W",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__Z-11W_WZ-11-ab0849ef.png",
            "id": "air_heli_z11w",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 170,
            "destroyed": 3,
            "timeIn": 0
        },
        {
            "type": "Helicopter",
            "vehicleName": "Mi-28 Havoc",
            "image": "https://cdn.gametools.network/vehicles/bf2042/ALX__Mi-28_Havoc-0d683246.png",
            "id": "air_heli_mi28",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 151,
            "spawns": 3,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 6934,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 3,
            "damageTo": 623,
            "destroyed": 6,
            "timeIn": 215
        },
        {
            "type": "Land",
            "vehicleName": "Panzer IV",
            "image": "https://cdn.gametools.network/vehicles/bf2042/GRA__Panzer-IV-3721bd56.png",
            "id": "sur_land_panziv",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 29,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 15
        },
        {
            "type": "Helicopter",
            "vehicleName": "KA-520 Super Hokum",
            "image": "https://cdn.gametools.network/vehicles/bf2042/Alligator-8db4b043.png",
            "id": "air_heli_ka52",
            "kills": 75,
            "killsPerMinute": 1.27,
            "damage": 17091,
            "spawns": 36,
            "roadKills": 0,
            "passengerAssists": 13,
            "multiKills": 22,
            "distanceTraveled": 107178,
            "driverAssists": 11,
            "vehiclesDestroyedWith": 11,
            "assists": 24,
            "callIns": 8,
            "damageTo": 3319,
            "destroyed": 23,
            "timeIn": 3540
        },
        {
            "type": "Land",
            "vehicleName": "T28",
            "image": "https://cdn.gametools.network/vehicles/bf2042/T14Armata-8890e549.png",
            "id": "sur_land_t28",
            "kills": 230,
            "killsPerMinute": 0.73,
            "damage": 40764,
            "spawns": 52,
            "roadKills": 1,
            "passengerAssists": 285,
            "multiKills": 33,
            "distanceTraveled": 74123,
            "driverAssists": 114,
            "vehiclesDestroyedWith": 18,
            "assists": 399,
            "callIns": 39,
            "damageTo": 6560,
            "destroyed": 35,
            "timeIn": 18925
        },
        {
            "type": "Land",
            "vehicleName": "CAV-BRAWLER",
            "image": "https://cdn.gametools.network/vehicles/bf2042/VEH_Icon_ArmoredCar_Patria-17d03f44.png",
            "id": "sur_land_brawler",
            "kills": 141,
            "killsPerMinute": 0.87,
            "damage": 16587,
            "spawns": 32,
            "roadKills": 20,
            "passengerAssists": 102,
            "multiKills": 34,
            "distanceTraveled": 66397,
            "driverAssists": 164,
            "vehiclesDestroyedWith": 7,
            "assists": 266,
            "callIns": 16,
            "damageTo": 4106,
            "destroyed": 30,
            "timeIn": 9770
        },
        {
            "type": "Stationary",
            "vehicleName": "9M133 Kornet",
            "image": "",
            "id": "sur_stat_kornet",
            "kills": 1,
            "killsPerMinute": 0,
            "damage": 132,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 23,
            "destroyed": 1,
            "timeIn": 0
        },
        {
            "type": "In-World",
            "vehicleName": "Polaris Sportsman",
            "image": "",
            "id": "sur_world_quadbike",
            "kills": 1,
            "killsPerMinute": 0.06,
            "damage": 100,
            "spawns": 0,
            "roadKills": 1,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 8553,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 206,
            "destroyed": 3,
            "timeIn": 1040
        },
        {
            "type": "In-World",
            "vehicleName": "Polaris RZR",
            "image": "",
            "id": "sur_world_rzrbuggy",
            "kills": 2,
            "killsPerMinute": 0.15,
            "damage": 195,
            "spawns": 0,
            "roadKills": 2,
            "passengerAssists": 0,
            "multiKills": 1,
            "distanceTraveled": 6152,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 1176,
            "destroyed": 12,
            "timeIn": 800
        },
        {
            "type": "Plane",
            "vehicleName": "Su-25TM Frogfoot",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__SU-25TM_Frogfoot-cba9e798.png",
            "id": "air_plane_su25",
            "kills": 3,
            "killsPerMinute": 0.37,
            "damage": 553,
            "spawns": 3,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 1,
            "distanceTraveled": 41115,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 2,
            "assists": 0,
            "callIns": 3,
            "damageTo": 13,
            "destroyed": 1,
            "timeIn": 490
        },
        {
            "type": "Helicopter",
            "vehicleName": "Mi-24 Hind",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__Mi-24SuperHind-faf17714.png",
            "id": "air_heli_mi24",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 189,
            "destroyed": 1,
            "timeIn": 0
        },
        {
            "type": "BMD-3",
            "image": "https://cdn.gametools.network/vehicles/bf2042/RUM__BMD-3_Bakhcha-86f10fe1.png",
            "id": "sur_land_bmd3",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 54,
            "destroyed": 1,
            "timeIn": 0
        },
        {
            "type": "In-World",
            "vehicleName": "Tuk-Tuk",
            "image": "",
            "id": "sur_world_tuktuk",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 1358,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 1,
            "timeIn": 230
        },
        {
            "type": "In-World",
            "vehicleName": "4x4 Utility",
            "image": "",
            "id": "sur_world_4x4",
            "kills": 1,
            "killsPerMinute": 0.06,
            "damage": 92,
            "spawns": 0,
            "roadKills": 1,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 8261,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 521,
            "destroyed": 6,
            "timeIn": 1015
        },
        {
            "type": "Helicopter",
            "vehicleName": "UAV-1",
            "image": "",
            "id": "air_heli_uav1",
            "kills": 2,
            "killsPerMinute": 0,
            "damage": 183,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 42,
            "destroyed": 2,
            "timeIn": 0
        },
        {
            "type": "Stationary",
            "vehicleName": "Flak 38",
            "image": "",
            "id": "sur_stat_flak38",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Stationary",
            "vehicleName": "HMG",
            "image": "",
            "id": "sur_stat_hmg",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "Stationary",
            "vehicleName": "KORD HMG",
            "image": "",
            "id": "sur_stat_kord",
            "kills": 1,
            "killsPerMinute": 0,
            "damage": 241,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 17,
            "destroyed": 2,
            "timeIn": 0
        },
        {
            "type": "Quadbike",
            "vehicleName": "Quadbike - BC2",
            "image": "",
            "id": "sur_world_quad_bc2",
            "kills": 0,
            "killsPerMinute": 0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 0,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 0
        },
        {
            "type": "RHIB",
            "vehicleName": "RHIB - BF3",
            "image": "",
            "id": "sur_world_rhib_bf3",
            "kills": 0,
            "killsPerMinute": 0.0,
            "damage": 0,
            "spawns": 0,
            "roadKills": 0,
            "passengerAssists": 0,
            "multiKills": 0,
            "distanceTraveled": 893,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 0,
            "destroyed": 0,
            "timeIn": 70
        }
    ],
    "weaponGroups": [
        {
            "groupName": "Primary",
            "id": "pri",
            "kills": 11948,
            "damage": 1454732,
            "bodyKills": 9158,
            "headshotKills": 2790,
            "hipfireKills": 2375,
            "multiKills": 358,
            "accuracy": "13.69%",
            "killsPerMinute": 0.77,
            "damagePerMinute": 93.9,
            "headshots": "23.35%",
            "hitVKills": 5.19,
            "shotsHit": 62064,
            "shotsFired": 453220,
            "spawns": 1145,
            "timeEquipped": 929545
        },
        {
            "groupName": "Assault Rifles",
            "id": "pri_ar",
            "kills": 4596,
            "damage": 567062,
            "bodyKills": 3626,
            "headshotKills": 970,
            "hipfireKills": 824,
            "multiKills": 55,
            "accuracy": "13.95%",
            "killsPerMinute": 0.76,
            "damagePerMinute": 93.96,
            "headshots": "21.11%",
            "hitVKills": 5.78,
            "shotsHit": 26575,
            "shotsFired": 190513,
            "spawns": 182,
            "timeEquipped": 362090
        },
        {
            "groupName": "Bolt Action",
            "id": "pri_ba",
            "kills": 572,
            "damage": 83067,
            "bodyKills": 287,
            "headshotKills": 285,
            "hipfireKills": 157,
            "multiKills": 5,
            "accuracy": "22.76%",
            "killsPerMinute": 0.4,
            "damagePerMinute": 58.0,
            "headshots": "49.83%",
            "hitVKills": 2.16,
            "shotsHit": 1237,
            "shotsFired": 5435,
            "spawns": 259,
            "timeEquipped": 85930
        },
        {
            "groupName": "DMR",
            "id": "pri_dmr",
            "kills": 1218,
            "damage": 148302,
            "bodyKills": 997,
            "headshotKills": 221,
            "hipfireKills": 106,
            "multiKills": 20,
            "accuracy": "16.55%",
            "killsPerMinute": 0.88,
            "damagePerMinute": 107.22,
            "headshots": "18.14%",
            "hitVKills": 4.15,
            "shotsHit": 5054,
            "shotsFired": 30532,
            "spawns": 68,
            "timeEquipped": 82990
        },
        {
            "groupName": "LMG",
            "id": "pri_lmg",
            "kills": 1477,
            "damage": 182164,
            "bodyKills": 1151,
            "headshotKills": 326,
            "hipfireKills": 203,
            "multiKills": 81,
            "accuracy": "8.42%",
            "killsPerMinute": 0.84,
            "damagePerMinute": 103.28,
            "headshots": "22.07%",
            "hitVKills": 5.98,
            "shotsHit": 8833,
            "shotsFired": 104847,
            "spawns": 101,
            "timeEquipped": 105830
        },
        {
            "groupName": "PDW",
            "id": "pri_pdw",
            "kills": 2687,
            "damage": 304478,
            "bodyKills": 2310,
            "headshotKills": 377,
            "hipfireKills": 602,
            "multiKills": 153,
            "accuracy": "14.71%",
            "killsPerMinute": 0.8,
            "damagePerMinute": 90.76,
            "headshots": "14.03%",
            "hitVKills": 5.67,
            "shotsHit": 15233,
            "shotsFired": 103524,
            "spawns": 510,
            "timeEquipped": 201275
        },
        {
            "groupName": "Shotguns",
            "id": "pri_misc_sg",
            "kills": 532,
            "damage": 58952,
            "bodyKills": 483,
            "headshotKills": 49,
            "hipfireKills": 445,
            "multiKills": 22,
            "accuracy": "44.62%",
            "killsPerMinute": 1.17,
            "damagePerMinute": 129.23,
            "headshots": "9.21%",
            "hitVKills": 5.72,
            "shotsHit": 3042,
            "shotsFired": 6818,
            "spawns": 22,
            "timeEquipped": 27370
        },
        {
            "groupName": "Railguns",
            "id": "pri_misc_rg",
            "kills": 811,
            "damage": 104600,
            "bodyKills": 256,
            "headshotKills": 555,
            "hipfireKills": 6,
            "multiKills": 22,
            "accuracy": "17.53%",
            "killsPerMinute": 0.85,
            "damagePerMinute": 109.24,
            "headshots": "68.43%",
            "hitVKills": 2.41,
            "shotsHit": 1955,
            "shotsFired": 11154,
            "spawns": 0,
            "timeEquipped": 57450
        },
        {
            "groupName": "Lever-Action Carbines",
            "id": "pri_misc_lac",
            "kills": 8,
            "damage": 1010,
            "bodyKills": 5,
            "headshotKills": 3,
            "hipfireKills": 2,
            "multiKills": 0,
            "accuracy": "36.96%",
            "killsPerMinute": 1.09,
            "damagePerMinute": 137.73,
            "headshots": "37.5%",
            "hitVKills": 2.12,
            "shotsHit": 17,
            "shotsFired": 46,
            "spawns": 3,
            "timeEquipped": 440
        },
        {
            "groupName": "Crossbows",
            "id": "pri_misc_cb",
            "kills": 47,
            "damage": 5097,
            "bodyKills": 43,
            "headshotKills": 4,
            "hipfireKills": 30,
            "multiKills": 0,
            "accuracy": "33.62%",
            "killsPerMinute": 0.46,
            "damagePerMinute": 49.57,
            "headshots": "8.51%",
            "hitVKills": 2.51,
            "shotsHit": 118,
            "shotsFired": 351,
            "spawns": 0,
            "timeEquipped": 6170
        },
        {
            "groupName": "Sidearm",
            "id": "sa",
            "kills": 573,
            "damage": 57979,
            "bodyKills": 461,
            "headshotKills": 112,
            "hipfireKills": 222,
            "multiKills": 30,
            "accuracy": "18.81%",
            "killsPerMinute": 1.58,
            "damagePerMinute": 160.13,
            "headshots": "19.55%",
            "hitVKills": 3.54,
            "shotsHit": 2027,
            "shotsFired": 10774,
            "spawns": 1145,
            "timeEquipped": 21725
        },
        {
            "groupName": "Miscellaneous",
            "id": "pri_misc",
            "kills": 1398,
            "damage": 169659,
            "bodyKills": 787,
            "headshotKills": 611,
            "hipfireKills": 483,
            "multiKills": 44,
            "accuracy": "27.94%",
            "killsPerMinute": 0.92,
            "damagePerMinute": 111.34,
            "headshots": "43.71%",
            "hitVKills": 3.67,
            "shotsHit": 5132,
            "shotsFired": 18369,
            "spawns": 25,
            "timeEquipped": 91430
        }
    ],
    "vehicleGroups": [
        {
            "groupName": "Air",
            "id": "air",
            "kills": 599,
            "killsPerMinute": 0.29,
            "damage": 113258,
            "spawns": 510,
            "roadKills": 7,
            "passengerAssists": 440,
            "multiKills": 153,
            "distanceTraveled": 5134125,
            "driverAssists": 401,
            "vehiclesDestroyedWith": 110,
            "assists": 841,
            "callIns": 308,
            "damageTo": 40004,
            "destroyed": 264,
            "timeIn": 125730
        },
        {
            "groupName": "Helicopter",
            "id": "air_heli",
            "kills": 531,
            "killsPerMinute": 0.29,
            "damage": 97005,
            "spawns": 374,
            "roadKills": 3,
            "passengerAssists": 440,
            "multiKills": 141,
            "distanceTraveled": 3865542,
            "driverAssists": 401,
            "vehiclesDestroyedWith": 62,
            "assists": 841,
            "callIns": 216,
            "damageTo": 30216,
            "destroyed": 205,
            "timeIn": 109655
        },
        {
            "groupName": "Plane",
            "id": "air_plane",
            "kills": 68,
            "killsPerMinute": 0.25,
            "damage": 16253,
            "spawns": 136,
            "roadKills": 4,
            "passengerAssists": 0,
            "multiKills": 12,
            "distanceTraveled": 1268583,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 48,
            "assists": 0,
            "callIns": 92,
            "damageTo": 9788,
            "destroyed": 59,
            "timeIn": 16075
        },
        {
            "groupName": "Sur",
            "id": "sur",
            "kills": 2014,
            "killsPerMinute": 0.44,
            "damage": 341326,
            "spawns": 548,
            "roadKills": 110,
            "passengerAssists": 1465,
            "multiKills": 449,
            "distanceTraveled": 823494,
            "driverAssists": 1105,
            "vehiclesDestroyedWith": 299,
            "assists": 2570,
            "callIns": 401,
            "damageTo": 59561,
            "destroyed": 405,
            "timeIn": 273260
        },
        {
            "groupName": "In-World",
            "id": "sur_world",
            "kills": 4,
            "killsPerMinute": 0.08,
            "damage": 387,
            "spawns": 0,
            "roadKills": 4,
            "passengerAssists": 0,
            "multiKills": 1,
            "distanceTraveled": 25217,
            "driverAssists": 0,
            "vehiclesDestroyedWith": 0,
            "assists": 0,
            "callIns": 0,
            "damageTo": 1903,
            "destroyed": 22,
            "timeIn": 3155
        },
        {
            "groupName": "Land",
            "id": "sur_land",
            "kills": 1748,
            "killsPerMinute": 0.42,
            "damage": 298361,
            "spawns": 481,
            "roadKills": 90,
            "passengerAssists": 1146,
            "multiKills": 356,
            "distanceTraveled": 700404,
            "driverAssists": 812,
            "vehiclesDestroyedWith": 283,
            "assists": 1958,
            "callIns": 344,
            "damageTo": 48969,
            "destroyed": 313,
            "timeIn": 252150
        },
        {
            "groupName": "Amphibious",
            "id": "sur_amph",
            "kills": 252,
            "killsPerMinute": 0.84,
            "damage": 40334,
            "spawns": 67,
            "roadKills": 16,
            "passengerAssists": 319,
            "multiKills": 91,
            "distanceTraveled": 97873,
            "driverAssists": 293,
            "vehiclesDestroyedWith": 10,
            "assists": 612,
            "callIns": 57,
            "damageTo": 7852,
            "destroyed": 59,
            "timeIn": 17955
        }
    ],
    "classes": [
        {
            "className": "Assault",
            "characterName": "Mackay",
            "statName": "Mackay (Assault)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconAlpha-db5e8361.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharAlpha_M-9377c351.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharAlpha_M-9377c351.png"
            },
            "id": "ass_alpha",
            "kills": 3561,
            "deaths": 2197,
            "kpm": 0.98,
            "killDeath": 1.62,
            "spawns": 14,
            "revives": 272,
            "assists": 2525,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 218040
        },
        {
            "className": "Support",
            "characterName": "Angel",
            "statName": "Angel (Support)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconIndia-d14545f9.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharIndia_M-354f160a.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharIndia_M-354f160a.png"
            },
            "id": "sup_india",
            "kills": 3125,
            "deaths": 1343,
            "kpm": 1.05,
            "killDeath": 2.33,
            "spawns": 534,
            "revives": 603,
            "assists": 1538,
            "hazardZoneStreaks": None,
            "secondsPlayed": 178355
        },
        {
            "className": "Support",
            "characterName": "Falck",
            "statName": "Falck (Support)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconFoxtrot-18ea6540.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharFoxtrot_M-318f1124.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharFoxtrot_M-318f1124.png"
            },
            "id": "sup_foxtrot",
            "kills": 4173,
            "deaths": 2514,
            "kpm": 0.95,
            "killDeath": 1.66,
            "spawns": 374,
            "revives": 1947,
            "assists": 3196,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 263865
        },
        {
            "className": "Recon",
            "characterName": "Paik",
            "statName": "Paik (Recon)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconEcho-891fd77b.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharEcho_M-38f70a40.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharEcho_M-38f70a40.png"
            },
            "id": "rec_echo",
            "kills": 575,
            "deaths": 305,
            "kpm": 1.28,
            "killDeath": 1.89,
            "spawns": 18,
            "revives": 37,
            "assists": 819,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 26850
        },
        {
            "className": "Assault",
            "characterName": "Sundance",
            "statName": "Sundance (Assault)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconCharlie-d3876e21.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharCharlie_M-b95efa1a.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharCharlie_M-b95efa1a.png"
            },
            "id": "ass_charlie",
            "kills": 792,
            "deaths": 598,
            "kpm": 0.91,
            "killDeath": 1.32,
            "spawns": 105,
            "revives": 68,
            "assists": 575,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 52455
        },
        {
            "className": "Assault",
            "characterName": "Dozer",
            "statName": "Dozer (Assault)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconDelta-7252512e.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharDelta_M-7a2bb7f1.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharDelta_M-7a2bb7f1.png"
            },
            "id": "ass_delta",
            "kills": 144,
            "deaths": 93,
            "kpm": 0.87,
            "killDeath": 1.55,
            "spawns": 32,
            "revives": 14,
            "assists": 56,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 9925
        },
        {
            "className": "Recon",
            "characterName": "Rao",
            "statName": "Rao (Recon)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconLima-1ba343fb.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharLima_M-8478cc5b.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharLima_M-8478cc5b.png"
            },
            "id": "rec_lima",
            "kills": 128,
            "deaths": 73,
            "kpm": 0.53,
            "killDeath": 1.75,
            "spawns": 0,
            "revives": 15,
            "assists": 155,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 14455
        },
        {
            "className": "Engineer",
            "characterName": "Lis",
            "statName": "Lis (Engineer)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconHotel-f243ed4d.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharHotel_M-702acec0.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharHotel_M-702acec0.png"
            },
            "id": "eng_hotel",
            "kills": 286,
            "deaths": 220,
            "kpm": 0.43,
            "killDeath": 1.3,
            "spawns": 29,
            "revives": 14,
            "assists": 405,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 39885
        },
        {
            "className": "Engineer",
            "characterName": "Irish",
            "statName": "Irish (Engineer)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconBravo-2196c05d.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharBravo_M-aa905ed8.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharBravo_M-aa905ed8.png"
            },
            "id": "eng_bravo",
            "kills": 819,
            "deaths": 212,
            "kpm": 0.3,
            "killDeath": 3.86,
            "spawns": 24,
            "revives": 63,
            "assists": 478,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 162615
        },
        {
            "className": "Support",
            "characterName": "Crawford",
            "statName": "Crawford (Support)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconMike-dc9fa665.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharAlpha_M-9377c351.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharAlpha_M-9377c351.png"
            },
            "avatarImage": "https://cdn.gametools.network/classes/bf2042/CharMike_M-726f8a53.png",
            "id": "sup_mike",
            "kills": 216,
            "deaths": 175,
            "kpm": 0.45,
            "killDeath": 1.23,
            "spawns": 0,
            "revives": 15,
            "assists": 382,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 29095
        },
        {
            "className": "Engineer",
            "characterName": "Boris",
            "statName": "Boris (Engineer)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconJuliet-3656a6da.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharJuliet_M-4ae561fa.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharJuliet_M-4ae561fa.png"
            },
            "id": "eng_juliet",
            "kills": 247,
            "deaths": 183,
            "kpm": 0.56,
            "killDeath": 1.35,
            "spawns": 3,
            "revives": 18,
            "assists": 311,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 26570
        },
        {
            "className": "Assault",
            "characterName": "Zain",
            "statName": "Zain (Assault)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconNovember-f3251048.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharNovember_M-4cb65528.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharNovember_M-4cb65528.png"
            },
            "id": "ass_november",
            "kills": 1958,
            "deaths": 1028,
            "kpm": 1.17,
            "killDeath": 1.9,
            "spawns": 0,
            "revives": 150,
            "assists": 1182,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 100195
        },
        {
            "className": "Recon",
            "characterName": "Casper",
            "statName": "Casper (Recon)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconKilo-9ffbea70.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharKilo_M-0e19f3d5.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharKilo_M-0e19f3d5.png"
            },
            "id": "rec_kilo",
            "kills": 211,
            "deaths": 100,
            "kpm": 0.81,
            "killDeath": 2.11,
            "spawns": 9,
            "revives": 10,
            "assists": 360,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 15575
        },
        {
            "className": "Recon",
            "characterName": "Blasco",
            "statName": "Blasco (Recon)",
            "image": "https://cdn.gametools.network/classes/bf2042/IconOscar-2214e1f3.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/CharOscar_M-76506937.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/CharOscar_M-76506937.png"
            },
            "id": "rec_oscar",
            "kills": 572,
            "deaths": 386,
            "kpm": 1.03,
            "killDeath": 1.48,
            "spawns": 0,
            "revives": 52,
            "assists": 732,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 33240
        },
        {
            "className": "Recon",
            "characterName": "BF3 Recon",
            "statName": "BF3 Recon",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassALX_Recon-c0b9e8d5.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/ALX_US_Recon_M-7e71219e.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/ALX_RU_Recon_M-78055d43.png"
            },
            "id": "rec_bf3",
            "kills": 4,
            "deaths": 5,
            "kpm": 0.41,
            "killDeath": 0.8,
            "spawns": 0,
            "revives": 1,
            "assists": 12,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 585
        },
        {
            "className": "Assault",
            "characterName": "BF3 Assault",
            "statName": "BF3 Assault",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassALX_Assault-af6b9941.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/ALX_US_Assault_M-f7f337bb.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/ALX_RU_Assault_M-fd459f1e.png"
            },
            "id": "ass_bf3",
            "kills": 43,
            "deaths": 38,
            "kpm": 1.03,
            "killDeath": 1.13,
            "spawns": 0,
            "revives": 0,
            "assists": 21,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 2500
        },
        {
            "className": "Support",
            "characterName": "BF3 Support",
            "statName": "BF3 Support",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassALX_Support-ecdaca86.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/ALX_US_Support_M-9ee7f351.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/ALX_RU_Support_M-eaf297c9.png"
            },
            "id": "sup_bf3",
            "kills": 5,
            "deaths": 10,
            "kpm": 0.08,
            "killDeath": 0.5,
            "spawns": 0,
            "revives": 0,
            "assists": 1,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 3640
        },
        {
            "className": "Engineer",
            "characterName": "BF3 Engineer",
            "statName": "BF3 Engineer",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassALX_Engineer-4a92d87f.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/ALX_US_Engineer_M-d987cde1.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/ALX_RU_Engineer_M-aff3c210.png"
            },
            "id": "eng_bf3",
            "kills": 26,
            "deaths": 9,
            "kpm": 2.4,
            "killDeath": 2.89,
            "spawns": 0,
            "revives": 3,
            "assists": 12,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 650
        },
        {
            "className": "Recon",
            "characterName": "BC2 Recon",
            "statName": "BC2 Recon",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassRUM_Recon-d3d2638f.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/RUM_US_Recon_M-eed05b82.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/RUM_RU_Recon_M-62b15dcf.png"
            },
            "id": "rec_bc2",
            "kills": 15,
            "deaths": 11,
            "kpm": 0.83,
            "killDeath": 1.36,
            "spawns": 2,
            "revives": 0,
            "assists": 30,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 1080
        },
        {
            "className": "Medic",
            "characterName": "BC2 Medic",
            "statName": "BC2 Medic",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassRUM_Medic-bb51e293.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/RUM_US_Medic_M-5c1d3f68.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/RUM_RU_Medic_M-91409f1f.png"
            },
            "id": "sup_bc2med",
            "kills": 0,
            "deaths": 0,
            "kpm": 0.0,
            "killDeath": 0.0,
            "spawns": 0,
            "revives": 0,
            "assists": 0,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 0
        },
        {
            "className": "Engineer",
            "characterName": "BC2 Engineer",
            "statName": "BC2 Engineer",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassRUM_Engineer-4632bd24.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/RUM_US_Engineer_M-58699cad.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/RUM_RU_Engineer_M-37754032.png"
            },
            "id": "eng_bc2",
            "kills": 3,
            "deaths": 5,
            "kpm": 0.55,
            "killDeath": 0.6,
            "spawns": 1,
            "revives": 1,
            "assists": 8,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 325
        },
        {
            "className": "Assault",
            "characterName": "BC2 Assault",
            "statName": "BC2 Assault",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassRUM_Assault-2cd51131.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/RUM_US_Assult_M-b1201e75.png",
                "rus": "https://cdn.gametools.network/classes/bf2042/RUM_RU_Assult_M-5b38d4f4.png"
            },
            "id": "ass_bc2",
            "kills": 9,
            "deaths": 4,
            "kpm": 1.93,
            "killDeath": 2.25,
            "spawns": 0,
            "revives": 0,
            "assists": 3,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 280
        },
        {
            "className": "Anti-tank",
            "characterName": "1942 Anti-tank",
            "statName": "1942 Anti-tank",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassGRA_AntiTank-fbfa815a.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/GRA_US_AntiTank_M-7d4b0bd5.png",
                "uk": "https://cdn.gametools.network/classes/bf2042/GRA_UK_AntiTank_M-ae283256.png",
                "ger": "https://cdn.gametools.network/classes/bf2042/GRA_GER_AntiTank_M-16f931ab.png"
            },
            "id": "sup_1942med",
            "kills": 15,
            "deaths": 13,
            "kpm": 1.24,
            "killDeath": 1.15,
            "spawns": 0,
            "revives": 0,
            "assists": 4,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 725
        },
        {
            "className": "Assault",
            "characterName": "1942 Assault",
            "statName": "1942 Assault",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassGRA_Assault-655085f0.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/GRA_US_Assault_M-2064c4a2.png",
                "uk": "https://cdn.gametools.network/classes/bf2042/GRA_UK_Assault_M-ab2e7b9b.png",
                "ger": "https://cdn.gametools.network/classes/bf2042/GRA_GER_Assault_M-4081d592.png"
            },
            "id": "ass_1942",
            "kills": 16,
            "deaths": 10,
            "kpm": 0.81,
            "killDeath": 1.6,
            "spawns": 0,
            "revives": 1,
            "assists": 10,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 1180
        },
        {
            "className": "Engineer",
            "characterName": "1942 Engineer",
            "statName": "1942 Engineer",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassGRA_Engineer-d40f1026.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/GRA_US_Engineer_M-f0e1f883.png",
                "uk": "https://cdn.gametools.network/classes/bf2042/GRA_UK_Engineer_M-239fabb2.png",
                "ger": "https://cdn.gametools.network/classes/bf2042/GRA_GER_Engineer_M-5658b780.png"
            },
            "id": "eng_1942",
            "kills": 0,
            "deaths": 0,
            "kpm": 0.0,
            "killDeath": 0.0,
            "spawns": 0,
            "revives": 0,
            "assists": 0,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 0
        },
        {
            "className": "Medic",
            "characterName": "1942 Medic",
            "statName": "1942 Medic",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassGRA_Medic-d7b0917e.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/GRA_US_Medic_M-a5ec1703.png",
                "uk": "https://cdn.gametools.network/classes/bf2042/GRA_UK_Medic_M-12bbb5d0.png",
                "ger": "https://cdn.gametools.network/classes/bf2042/GRA_GER_Medic_M-d8b825cd.png"
            },
            "id": "eng_1942at",
            "kills": 0,
            "deaths": 0,
            "kpm": 0.0,
            "killDeath": 0.0,
            "spawns": 0,
            "revives": 0,
            "assists": 0,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 0
        },
        {
            "className": "Scout",
            "characterName": "1942 Scout",
            "statName": "1942 Scout",
            "image": "https://cdn.gametools.network/classes/bf2042/iconClassGRA_Scout-5c77e572.svg",
            "avatarImages": {
                "us": "https://cdn.gametools.network/classes/bf2042/GRA_US_Scout_M-63460ca3.png",
                "uk": "https://cdn.gametools.network/classes/bf2042/GRA_UK_Scout_M-1c4c3bfe.png",
                "ger": "https://cdn.gametools.network/classes/bf2042/GRA_GER_Scout_M-2671d6da.png"
            },
            "id": "rec_1942sct",
            "kills": 5,
            "deaths": 3,
            "kpm": 0.53,
            "killDeath": 1.67,
            "spawns": 0,
            "revives": 0,
            "assists": 1,
            "hazardZoneStreaks": 0,
            "secondsPlayed": 565
        }
    ],
    "gamemodes": [
        {
            "gamemodeName": "Breakthrough Large",
            "image": "https://portal.battlefield.com/4141963/assets/icons/Icon_Breakthrough.svg",
            "id": "bt",
            "kills": 5282,
            "assists": 5047,
            "revives": 1539,
            "bestSquad": 27,
            "wins": 145,
            "losses": 153,
            "mvp": 11,
            "matches": 298,
            "sectorDefend": 0,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 876,
            "objectivesCaptured": 185,
            "objectivesDestroyed": 0,
            "objetiveTime": 41100,
            "kpm": 0.83,
            "winPercent": "48.66%",
            "secondsPlayed": 380275
        },
        {
            "gamemodeName": "Breakthrough",
            "image": "https://portal.battlefield.com/4141963/assets/icons/Icon_Breakthrough.svg",
            "id": "btsmall",
            "kills": 2691,
            "assists": 1963,
            "revives": 743,
            "bestSquad": 12,
            "wins": 74,
            "losses": 73,
            "mvp": 5,
            "matches": 147,
            "sectorDefend": 0,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 391,
            "objectivesCaptured": 89,
            "objectivesDestroyed": 0,
            "objetiveTime": 18760,
            "kpm": 0.92,
            "winPercent": "50.34%",
            "secondsPlayed": 175915
        },
        {
            "gamemodeName": "Conquest Large",
            "image": "https://portal.battlefield.com/3396060/assets/icons/Icon_Conquest.svg",
            "id": "cq",
            "kills": 6936,
            "assists": 5932,
            "revives": 1260,
            "bestSquad": 56,
            "wins": 217,
            "losses": 213,
            "mvp": 11,
            "matches": 430,
            "sectorDefend": 3777,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 1042,
            "objectivesCaptured": 472,
            "objectivesDestroyed": 0,
            "objetiveTime": 69150,
            "kpm": 0.8,
            "winPercent": "50.47%",
            "secondsPlayed": 520890
        },
        {
            "gamemodeName": "Conquest",
            "image": "https://portal.battlefield.com/3396060/assets/icons/Icon_Conquest_Small.svg",
            "id": "cqsmall",
            "kills": 3381,
            "assists": 3341,
            "revives": 548,
            "bestSquad": 38,
            "wins": 101,
            "losses": 96,
            "mvp": 8,
            "matches": 197,
            "sectorDefend": 2033,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 492,
            "objectivesCaptured": 237,
            "objectivesDestroyed": 0,
            "objetiveTime": 29095,
            "kpm": 0.87,
            "winPercent": "51.27%",
            "secondsPlayed": 232015
        },
        {
            "gamemodeName": "Custom",
            "image": "https://portal.battlefield.com/3396060/assets/icons/custom_wrench_hammer.svg",
            "id": "modbuilder",
            "kills": 1178,
            "assists": 706,
            "revives": 212,
            "bestSquad": 34,
            "wins": 102,
            "losses": 105,
            "mvp": 21,
            "matches": 208,
            "sectorDefend": 0,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 43,
            "objectivesCaptured": 5657,
            "objectivesDestroyed": 0,
            "objetiveTime": 40170,
            "kpm": 0.46,
            "winPercent": "49.28%",
            "secondsPlayed": 152115
        },
        {
            "gamemodeName": "Rush",
            "image": "https://portal.battlefield.com/3396060/assets/icons/Icon_Rush.svg",
            "id": "rush",
            "kills": 2656,
            "assists": 782,
            "revives": 153,
            "bestSquad": 12,
            "wins": 32,
            "losses": 35,
            "mvp": 8,
            "matches": 67,
            "sectorDefend": 0,
            "objectivesArmed": 57,
            "objectivesDisarmed": 24,
            "objectivesDefended": 0,
            "objectivesCaptured": 0,
            "objectivesDestroyed": 6,
            "objetiveTime": 0,
            "kpm": 0.84,
            "winPercent": "47.76%",
            "secondsPlayed": 190080
        },
        {
            "gamemodeName": "Hazard Zone Large",
            "image": "https://cdn.gametools.network/hazard-zone.jpg",
            "id": "hz",
            "kills": 77,
            "assists": 30,
            "revives": 16,
            "bestSquad": 38,
            "wins": 0,
            "losses": 11,
            "mvp": 0,
            "matches": 11,
            "sectorDefend": 0,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 0,
            "objectivesCaptured": 0,
            "objectivesDestroyed": 0,
            "objetiveTime": 0,
            "kpm": 0.5,
            "winPercent": "0.0%",
            "secondsPlayed": 9325
        },
        {
            "gamemodeName": "Hazard Zone",
            "image": "https://cdn.gametools.network/hazard-zone.jpg",
            "id": "hzsmall",
            "kills": 0,
            "assists": 0,
            "revives": 0,
            "bestSquad": 0,
            "wins": 0,
            "losses": 0,
            "mvp": 0,
            "matches": 0,
            "sectorDefend": 0,
            "objectivesArmed": 0,
            "objectivesDisarmed": 0,
            "objectivesDefended": 0,
            "objectivesCaptured": 0,
            "objectivesDestroyed": 0,
            "objetiveTime": 0,
            "kpm": 0.0,
            "winPercent": "0.0%",
            "secondsPlayed": 0
        }
    ],
    "maps": [
        {
            "mapName": "Renewal",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/ad/09/Map_Art_BF2042_TW_L-ad09f15f.png",
            "id": "kin_renewal",
            "wins": 36,
            "losses": 30,
            "matches": 66,
            "winPercent": "54.55%",
            "secondsPlayed": 90945
        },
        {
            "mapName": "Orbital",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/d1/5a/Map_Art_BF2042_ORB_L-d15a84f6.png",
            "id": "kin_orbital",
            "wins": 29,
            "losses": 37,
            "matches": 66,
            "winPercent": "43.94%",
            "secondsPlayed": 74755
        },
        {
            "mapName": "Manifest",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/22/b3/Map_Art_BF2042_LH_L-22b32735.png",
            "id": "kin_manifest",
            "wins": 37,
            "losses": 35,
            "matches": 72,
            "winPercent": "51.39%",
            "secondsPlayed": 84530
        },
        {
            "mapName": "Discarded",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/76/46/Map_Art_BF2042_DIS_L-764694ca.png",
            "id": "kin_discarded",
            "wins": 29,
            "losses": 36,
            "matches": 65,
            "winPercent": "44.62%",
            "secondsPlayed": 142960
        },
        {
            "mapName": "Kaleidescope",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/6b/a7/Map_Art_BF2042_KS_L-6ba702e0.png",
            "id": "kin_kaleidescope",
            "wins": 26,
            "losses": 37,
            "matches": 63,
            "winPercent": "41.27%",
            "secondsPlayed": 77325
        },
        {
            "mapName": "Breakaway",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/19/ee/Map_Art_BF2042_IRR_L-19eeef98.png",
            "id": "kin_breakaway",
            "wins": 21,
            "losses": 37,
            "matches": 58,
            "winPercent": "36.21%",
            "secondsPlayed": 65210
        },
        {
            "mapName": "Hourglass",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/95/35/Map_Art_BF2042_HG_L-9535453c.png",
            "id": "kin_hourglass",
            "wins": 26,
            "losses": 37,
            "matches": 63,
            "winPercent": "41.27%",
            "secondsPlayed": 75625
        },
        {
            "mapName": "Spearhead",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/pyrite/images/Map_Art_BF2042_LO-ce4a8588.jpg",
            "id": "kin_lightsout",
            "wins": 60,
            "losses": 48,
            "matches": 108,
            "winPercent": "55.56%",
            "secondsPlayed": 124850
        },
        {
            "mapName": "Exposure",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/66/e8/Map_Art_BF2042_RID_L-66e845a2.png",
            "id": "kin_exposure",
            "wins": 32,
            "losses": 36,
            "matches": 68,
            "winPercent": "47.06%",
            "secondsPlayed": 80460
        },
        {
            "mapName": "Stranded",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/d7/5f/Map_Art_BF2042_DRA_L-d75f98a0.png",
            "id": "kin_drained",
            "wins": 37,
            "losses": 51,
            "matches": 88,
            "winPercent": "42.05%",
            "secondsPlayed": 85785
        },
        {
            "mapName": "Noshahr Canals",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/41/a1/Map_Art_BF3_NC_L-41a1e94f.png",
            "id": "mica_noshahr",
            "wins": 14,
            "losses": 9,
            "matches": 23,
            "winPercent": "60.87%",
            "secondsPlayed": 16110
        },
        {
            "mapName": "Caspian Border",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/df/50/Map_Art_BF3_CB_L-df505a1f.png",
            "id": "mica_caspian",
            "wins": 56,
            "losses": 57,
            "matches": 113,
            "winPercent": "49.56%",
            "secondsPlayed": 96900
        },
        {
            "mapName": "Valparaiso",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/60/03/Map_Art_BFBC2_VP_L-600385e2.png",
            "id": "mica_valparaiso",
            "wins": 18,
            "losses": 11,
            "matches": 29,
            "winPercent": "62.07%",
            "secondsPlayed": 31240
        },
        {
            "mapName": "Arica Harbor",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/60/b4/Map_Art_BFBC2_AH_L-60b49760.png",
            "id": "mica_aricaharbor",
            "wins": 21,
            "losses": 14,
            "matches": 35,
            "winPercent": "60.0%",
            "secondsPlayed": 30195
        },
        {
            "mapName": "Battle of the Bulge",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/25/59/Map_Art_BF1942_BB_L-2559980c.png",
            "id": "mica_bulge",
            "wins": 4,
            "losses": 7,
            "matches": 11,
            "winPercent": "36.36%",
            "secondsPlayed": 16830
        },
        {
            "mapName": "El Alamein",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/gamedata/kingston/d7/d3/Map_Art_BF1942_EA_L-d7d3186f.png",
            "id": "mica_elalamein",
            "wins": 4,
            "losses": 5,
            "matches": 10,
            "winPercent": "44.44%",
            "secondsPlayed": 53065
        },
        {
            "mapName": "Flashpoint",
            "image": "https://eaassets-a.akamaihd.net/battlelog/battlebinary/pyrite/images/Map_Art_BF2042_BOL-215cc9e7.jpg",
            "id": "kin_flashpoint",
            "wins": 34,
            "losses": 20,
            "matches": 54,
            "winPercent": "62.96%",
            "secondsPlayed": 57105
        },
        {
            "mapName": "Reclaimed",
            "image": "http://eaassets-a.akamaihd.net/battlelog/battlebinary/pyrite/images/Map_Art_BF2042_SC-91339cd1.jpg",
            "id": "kin_reclaimed",
            "wins": 29,
            "losses": 16,
            "matches": 45,
            "winPercent": "64.44%",
            "secondsPlayed": 55530
        },
        {
            "mapName": "Redacted",
            "image": "http://eaassets-a.akamaihd.net/battlelog/battlebinary/pyrite/images/Map_Image_BF2042_SA-d546e7ce.png",
            "id": "kin_salem",
            "wins": 16,
            "losses": 11,
            "matches": 27,
            "winPercent": "59.26%",
            "secondsPlayed": 38055
        }
    ],
    "gadgets": [
        {
            "type": "Explosives",
            "gadgetName": "C5 Explosive [Kingston]",
            "image": "https://cdn.gametools.network/gadgets/bf2042/C4-6e2329da.png",
            "id": "open_equip_c5",
            "kills": 416,
            "spawns": 3040,
            "damage": 57706,
            "uses": 5139,
            "multiKills": 136,
            "vehiclesDestroyedWith": 157,
            "kpm": 1.59,
            "dpm": 220.32,
            "secondsPlayed": 15715
        },
        {
            "type": "Character",
            "gadgetName": "SOB-8 Ballistic Shield",
            "image": "https://cdn.gametools.network/gadgets/bf2042/DeployableCover-9a72c1fc.png",
            "id": "char_shield",
            "kills": 69,
            "spawns": 89,
            "damage": 5798,
            "uses": 436,
            "multiKills": 9,
            "vehiclesDestroyedWith": 0,
            "kpm": 1.61,
            "dpm": 135.36,
            "secondsPlayed": 2570
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "Carl Gustaf M5",
            "image": "https://cdn.gametools.network/gadgets/bf2042/CarlGustafMk4-460b101e.png",
            "id": "open_equip_rl_cgmk4",
            "kills": 97,
            "spawns": 562,
            "damage": 21253,
            "uses": 781,
            "multiKills": 14,
            "vehiclesDestroyedWith": 51,
            "kpm": 0.46,
            "dpm": 100.17,
            "secondsPlayed": 12730
        },
        {
            "type": "Grenade",
            "gadgetName": "Frag Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/FragGrenade-8eaf52c4.png",
            "id": "open_throw_gren_frag",
            "kills": 177,
            "spawns": 2008,
            "damage": 26642,
            "uses": 2466,
            "multiKills": 33,
            "vehiclesDestroyedWith": 21,
            "kpm": 1.8,
            "dpm": 270.71,
            "secondsPlayed": 5905
        },
        {
            "type": "Smart Explosives",
            "gadgetName": "Anti Armor",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SmartExplosive1-c93ac628.png",
            "id": "char_smart_antiarmor",
            "kills": 1,
            "spawns": 0,
            "damage": 727,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 8,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Smart Explosives",
            "gadgetName": "Scatter Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SmartExplosive3-02e833c6.png",
            "id": "char_smart_scatter",
            "kills": 73,
            "spawns": 0,
            "damage": 13565,
            "uses": 0,
            "multiKills": 17,
            "vehiclesDestroyedWith": 22,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Equipment",
            "gadgetName": "Repair Tool",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RepairTool-ed6ee81f.png",
            "id": "open_equip_repair",
            "kills": 1,
            "spawns": 65,
            "damage": 990,
            "uses": 57300,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 3.16,
            "secondsPlayed": 18780
        },
        {
            "type": "Character",
            "gadgetName": "SG-36 Sentry Gun",
            "image": "https://cdn.gametools.network/gadgets/bf2042/AutoTurret-94167140.png",
            "id": "char_sentrygun",
            "kills": 6,
            "spawns": 178,
            "damage": 1081,
            "uses": 194,
            "multiKills": 1,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.96,
            "dpm": 172.96,
            "secondsPlayed": 375
        },
        {
            "type": "Grenade",
            "gadgetName": "EMP Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/EMPGrenade-f5f1265e.png",
            "id": "open_throw_gren_emp",
            "kills": 1,
            "spawns": 356,
            "damage": 971,
            "uses": 626,
            "multiKills": 0,
            "vehiclesDestroyedWith": 16,
            "kpm": 0.03,
            "dpm": 32.01,
            "secondsPlayed": 1820
        },
        {
            "type": "Throwable",
            "gadgetName": "Anti-Tank Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_ICO_Throwable_AntiTank-9a518e3b.png",
            "id": "open_throw_gren_at",
            "kills": 5,
            "spawns": 71,
            "damage": 1579,
            "uses": 45,
            "multiKills": 1,
            "vehiclesDestroyedWith": 2,
            "kpm": 2.5,
            "dpm": 789.5,
            "secondsPlayed": 120
        },
        {
            "type": "Throwable",
            "gadgetName": "Mini Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_ICO_Throwable_Mini-30504512.png",
            "id": "open_throw_gren_mini",
            "kills": 0,
            "spawns": 20,
            "damage": 30,
            "uses": 35,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 25.71,
            "secondsPlayed": 70
        },
        {
            "type": "Ammo Box",
            "gadgetName": "Ammo Box BF3 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__AmmoBox-8a4f44a4.png",
            "id": "char_amm_alx",
            "kills": 0,
            "spawns": 11,
            "damage": 0,
            "uses": 369,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 325
        },
        {
            "type": "Ammo Box",
            "gadgetName": "Ammo Box BC2 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__Ammo-box-02b93059.png",
            "id": "char_amm_rum",
            "kills": 0,
            "spawns": 8,
            "damage": 0,
            "uses": 370,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 325
        },
        {
            "type": "Character",
            "gadgetName": "Grappling Hook",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GrappleGun-12f57257.png",
            "id": "char_grapple",
            "kills": 0,
            "spawns": 2124,
            "damage": 50,
            "uses": 1824,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.43,
            "secondsPlayed": 7015
        },
        {
            "type": "Character",
            "gadgetName": "Cyber Warfare Suite (Signal Hacker)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SignalHacker-78998885.png",
            "id": "char_hack",
            "kills": 0,
            "spawns": 73,
            "damage": 0,
            "uses": 126,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 1010
        },
        {
            "type": "Medkit",
            "gadgetName": "Medkit BF3 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__MedicBag-bb24ea02.png",
            "id": "char_med_alx",
            "kills": 0,
            "spawns": 14,
            "damage": 0,
            "uses": 10,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 10
        },
        {
            "type": "Medkit",
            "gadgetName": "Medkit BF1942 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__Medkit-1801e49b.png",
            "id": "char_med_gra",
            "kills": 0,
            "spawns": 21,
            "damage": 0,
            "uses": 23,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 15
        },
        {
            "type": "Medkit",
            "gadgetName": "Medkit BF2 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__MED-KIT-db0a4d20.png",
            "id": "char_med_rum",
            "kills": 0,
            "spawns": 8,
            "damage": 0,
            "uses": 10,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 10
        },
        {
            "type": "Character",
            "gadgetName": "Motion Sensor (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__Motion-Sensor-94ffc935.png",
            "id": "char_motion",
            "kills": 0,
            "spawns": 11,
            "damage": 0,
            "uses": 18,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 60
        },
        {
            "type": "Character",
            "gadgetName": "OV-P Recon Drone",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ReconDrone-025b3091.png",
            "id": "char_rdrone",
            "kills": 1,
            "spawns": 97,
            "damage": 590,
            "uses": 114,
            "multiKills": 0,
            "vehiclesDestroyedWith": 4,
            "kpm": 0.02,
            "dpm": 9.55,
            "secondsPlayed": 3705
        },
        {
            "type": "Repair Tool",
            "gadgetName": "Repair Tool BF3 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__RepairTool_BlowTorch-cf0d99f8.png",
            "id": "char_repair_alx",
            "kills": 0,
            "spawns": 9,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Repair Tool",
            "gadgetName": "Repair Tool BF1942 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__RepairTool_Wrench-0df0964f.png",
            "id": "char_repair_gra",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Repair Tool",
            "gadgetName": "Repair Tool BC2 (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__Repair-Tool-a9c7ee75.png",
            "id": "char_repair_rum",
            "kills": 0,
            "spawns": 5,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 5
        },
        {
            "type": "Character",
            "gadgetName": "S21 Syrette Pistol",
            "image": "https://cdn.gametools.network/gadgets/bf2042/StimPistol-79f246f6.png",
            "id": "char_syrettepistol",
            "kills": 0,
            "spawns": 2544,
            "damage": 340,
            "uses": 4709,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 1.53,
            "secondsPlayed": 13345
        },
        {
            "type": "Character",
            "gadgetName": "T-UGS (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__T-UGS-805c63da.png",
            "id": "char_tugs",
            "kills": 0,
            "spawns": 5,
            "damage": 0,
            "uses": 184,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 250
        },
        {
            "type": "Equipment",
            "gadgetName": "Ammo Crate",
            "image": "https://cdn.gametools.network/gadgets/bf2042/AmmoCrate-46f6060c.png",
            "id": "open_equip_ammocr",
            "kills": 0,
            "spawns": 2641,
            "damage": 0,
            "uses": 2818,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 2645
        },
        {
            "type": "Equipment",
            "gadgetName": "IBA Armor Plate",
            "image": "https://cdn.gametools.network/gadgets/bf2042/Armor-4f61bc6a.png",
            "id": "open_equip_armor",
            "kills": 0,
            "spawns": 176,
            "damage": 0,
            "uses": 189,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 345
        },
        {
            "type": "Explosives",
            "gadgetName": "C4 [Portal]",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__C4-560f90c9.png",
            "id": "open_equip_c4",
            "kills": 2,
            "spawns": 18,
            "damage": 281,
            "uses": 25,
            "multiKills": 0,
            "vehiclesDestroyedWith": 1,
            "kpm": 1.14,
            "dpm": 160.57,
            "secondsPlayed": 105
        },
        {
            "type": "Explosives",
            "gadgetName": "M18 Claymore",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__M18_Claymore-6e18b4dc.png",
            "id": "open_equip_claymore",
            "kills": 9,
            "spawns": 49,
            "damage": 895,
            "uses": 53,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 3.48,
            "dpm": 346.45,
            "secondsPlayed": 155
        },
        {
            "type": "Equipment",
            "gadgetName": "MAV",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__MAV-68f661d6.png",
            "id": "open_equip_mav",
            "kills": 0,
            "spawns": 3,
            "damage": 0,
            "uses": 7,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 50
        },
        {
            "type": "Equipment",
            "gadgetName": "Defibrillator",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__Defibrillator-a0645d72.png",
            "id": "open_equip_defib",
            "kills": 0,
            "spawns": 8,
            "damage": 0,
            "uses": 1001,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 2730
        },
        {
            "type": "Equipment",
            "gadgetName": "Defibrillator (BC2)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__Defibrillator-a0645d72.png",
            "id": "open_bc2_defib",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Equipment",
            "gadgetName": "Defibrillator (BF3)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__Defibrillator-a0645d72.png",
            "id": "open_bf3_defib",
            "kills": 0,
            "spawns": 6,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "FGM-148 Javelin",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__FGM-148_Javelin-9aa9a62e.png",
            "id": "open_equip_javelin",
            "kills": 28,
            "spawns": 55,
            "damage": 4839,
            "uses": 73,
            "multiKills": 13,
            "vehiclesDestroyedWith": 23,
            "kpm": 0.69,
            "dpm": 118.99,
            "secondsPlayed": 2440
        },
        {
            "type": "Equipment",
            "gadgetName": "Medical Crate",
            "image": "https://cdn.gametools.network/gadgets/bf2042/MedicCrate-7bb8b6dd.png",
            "id": "open_equip_medcr",
            "kills": 0,
            "spawns": 571,
            "damage": 0,
            "uses": 679,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 930
        },
        {
            "type": "Equipment",
            "gadgetName": "Med-Pen (Self-heal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SelfHeal-69f9e60e.png",
            "id": "open_equip_medpen",
            "kills": 0,
            "spawns": 92,
            "damage": 0,
            "uses": 1358,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 2600
        },
        {
            "type": "Explosives",
            "gadgetName": "AT Mine [Kingston]",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ATMine-b1dc1dbb.png",
            "id": "open_equip_mine_at",
            "kills": 15,
            "spawns": 140,
            "damage": 2136,
            "uses": 171,
            "multiKills": 7,
            "vehiclesDestroyedWith": 8,
            "kpm": 2.0,
            "dpm": 284.8,
            "secondsPlayed": 450
        },
        {
            "type": "Explosives",
            "gadgetName": "Mortar Strike",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__Mortar-Strike_Binoculars-a03f59a7.png",
            "id": "open_equip_mstrke",
            "kills": 5,
            "spawns": 8,
            "damage": 600,
            "uses": 5,
            "multiKills": 3,
            "vehiclesDestroyedWith": 1,
            "kpm": 4.0,
            "dpm": 480.0,
            "secondsPlayed": 75
        },
        {
            "type": "Equipment",
            "gadgetName": "Insertion Beacon (Spawn) (Portal)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SpawnBeacon-d9f22afe.png",
            "id": "open_equip_radiob",
            "kills": 0,
            "spawns": 2,
            "damage": 0,
            "uses": 1,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "RPG-7",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__RPG7-9e06e1f6.png",
            "id": "open_equip_rpg",
            "kills": 0,
            "spawns": 5,
            "damage": 137,
            "uses": 7,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 82.2,
            "secondsPlayed": 100
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "SMAW",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__SMAW-7a94c5a0.png",
            "id": "open_equip_smaw",
            "kills": 0,
            "spawns": 2,
            "damage": 83,
            "uses": 6,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 62.25,
            "secondsPlayed": 80
        },
        {
            "type": "Throwable",
            "gadgetName": "ExpPack",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__Dynamite-27dc00ae.png",
            "id": "open_throw_dyn",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Grenade",
            "gadgetName": "Concussion Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/FlashBang-ea224d5f.png",
            "id": "open_throw_gren_conc",
            "kills": 0,
            "spawns": 40,
            "damage": 34,
            "uses": 83,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 7.7,
            "secondsPlayed": 265
        },
        {
            "type": "Grenade",
            "gadgetName": "Incendiary Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/IncendiaryGrenade-ea934278.png",
            "id": "open_throw_gren_inc",
            "kills": 35,
            "spawns": 469,
            "damage": 7151,
            "uses": 538,
            "multiKills": 5,
            "vehiclesDestroyedWith": 1,
            "kpm": 1.48,
            "dpm": 302.15,
            "secondsPlayed": 1420
        },
        {
            "type": "Grenade",
            "gadgetName": "Smoke Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SmokeGrenade-ad6ed4ef.png",
            "id": "open_throw_gren_smo",
            "kills": 0,
            "spawns": 5512,
            "damage": 28,
            "uses": 4097,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.18,
            "secondsPlayed": 9155
        },
        {
            "type": "Throwable",
            "gadgetName": "Prox Sensor (Motion)",
            "image": "https://cdn.gametools.network/gadgets/bf2042/MotionMine-2766d34f.png",
            "id": "open_throw_motion",
            "kills": 0,
            "spawns": 716,
            "damage": 32,
            "uses": 1371,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.56,
            "secondsPlayed": 3435
        },
        {
            "type": "Throwable",
            "gadgetName": "Throwing Knife",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_Icon_throwingKnife-4b78bd1d.png",
            "id": "open_throw_knife",
            "kills": 14,
            "spawns": 261,
            "damage": 1233,
            "uses": 94,
            "multiKills": 3,
            "vehiclesDestroyedWith": 0,
            "kpm": 1.71,
            "dpm": 150.98,
            "secondsPlayed": 490
        },
        {
            "type": "Equipment",
            "gadgetName": "Smoke Grenade Launcher",
            "image": "https://cdn.gametools.network/gadgets/bf2042/SOFLAM-3c23636f.png",
            "id": "open_equip_soflam",
            "kills": 0,
            "spawns": 269,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 8825
        },
        {
            "type": "Explosives",
            "gadgetName": "Lindmine",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__AT-Mine-d4052b6a.png",
            "id": "open_1942_atmine",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Character",
            "gadgetName": "XM370A",
            "image": "https://cdn.gametools.network/gadgets/bf2042/icon_gadget_XM370A-2d6514a4.png",
            "id": "char_xm370a",
            "kills": 474,
            "spawns": 1007,
            "damage": 64537,
            "uses": 6248,
            "multiKills": 95,
            "vehiclesDestroyedWith": 61,
            "kpm": 1.41,
            "dpm": 192.22,
            "secondsPlayed": 20145
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "FXM-33 AA Missile",
            "image": "https://cdn.gametools.network/gadgets/bf2042/Stinger-630fbfbb.png",
            "id": "open_equip_stinger",
            "kills": 25,
            "spawns": 158,
            "damage": 12612,
            "uses": 160,
            "multiKills": 9,
            "vehiclesDestroyedWith": 23,
            "kpm": 0.25,
            "dpm": 124.97,
            "secondsPlayed": 6055
        },
        {
            "type": "Launchers",
            "gadgetName": "Smoke Grenade Launchers",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GL_SmokeGL-3ad2ef03.png",
            "id": "open_equip_smokegl",
            "kills": 0,
            "spawns": 34,
            "damage": 0,
            "uses": 61,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 280
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "M136 AT4",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RUM__M136AT4-dcac494a.png",
            "id": "open_equip_m136",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Character",
            "gadgetName": "G-84 TGM",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RL_HomingMissiles_TVGuided-2e52736c.png",
            "id": "char_missiles",
            "kills": 38,
            "spawns": 222,
            "damage": 8763,
            "uses": 458,
            "multiKills": 12,
            "vehiclesDestroyedWith": 20,
            "kpm": 0.26,
            "dpm": 61.07,
            "secondsPlayed": 8610
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "Bazooka",
            "image": "https://cdn.gametools.network/weapons/bf2042/GRA__Weapons_default_Bazooka-ce1225d1.png",
            "id": "open_equip_zook",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Rocker Launcher",
            "gadgetName": "SA-18 Igla",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__SA-18_IGLA-a09c38f1.png",
            "id": "open_equip_igla",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Character",
            "gadgetName": "Binoculars BF1942",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__Binoculars-33bc2957.png",
            "id": "char_bino",
            "kills": 0,
            "spawns": 3,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 10
        },
        {
            "type": "Supply Bag",
            "gadgetName": "Crate",
            "image": "",
            "id": "char_suppbag_crate",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 234,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Supply Bag",
            "gadgetName": "Pouch",
            "image": "",
            "id": "char_suppbag_pouch",
            "kills": 0,
            "spawns": 1375,
            "damage": 0,
            "uses": 1328,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 9135
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "Panzerschreck",
            "image": "",
            "id": "open_equip_shrek",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Equipment",
            "gadgetName": "Insertion Beacon (Spawn)",
            "image": "",
            "id": "open_equip_spawnb",
            "kills": 0,
            "spawns": 499,
            "damage": 0,
            "uses": 1542,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 2595
        },
        {
            "type": "Grenade",
            "gadgetName": "BF1942 Grenade",
            "image": "",
            "id": "open_throw_gren_1942",
            "kills": 1,
            "spawns": 26,
            "damage": 126,
            "uses": 28,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 1.2,
            "dpm": 151.2,
            "secondsPlayed": 50
        },
        {
            "type": "Grenade",
            "gadgetName": "BC2 Grenade",
            "image": "",
            "id": "open_throw_gren_bc2",
            "kills": 0,
            "spawns": 20,
            "damage": 31,
            "uses": 12,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 74.4,
            "secondsPlayed": 25
        },
        {
            "type": "Grenade",
            "gadgetName": "BF3 Grenade",
            "image": "",
            "id": "open_throw_gren_bf3",
            "kills": 2,
            "spawns": 39,
            "damage": 114,
            "uses": 24,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 2.4,
            "dpm": 136.8,
            "secondsPlayed": 50
        },
        {
            "type": "Character",
            "gadgetName": "Mounted Vulcan",
            "image": "https://cdn.gametools.network/gadgets/bf2042/RL_Placed_Minigun-f9b2c5bf.png",
            "id": "char_staticminigun",
            "kills": 8,
            "spawns": 166,
            "damage": 1518,
            "uses": 7206,
            "multiKills": 0,
            "vehiclesDestroyedWith": 2,
            "kpm": 2.04,
            "dpm": 387.57,
            "secondsPlayed": 235
        },
        {
            "type": "Rocket Launcher",
            "gadgetName": "RPG-7v2",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_ICO_Launcher_RPG-30dde618.png",
            "id": "open_equip_rpgv2",
            "kills": 15,
            "spawns": 81,
            "damage": 2150,
            "uses": 109,
            "multiKills": 2,
            "vehiclesDestroyedWith": 5,
            "kpm": 0.86,
            "dpm": 122.86,
            "secondsPlayed": 1050
        },
        {
            "type": "Grenade",
            "gadgetName": "SPH Explosive Launcher",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_ICO_Open_StickyGrenade-1e3ea066.png",
            "id": "open_equip_stickygl",
            "kills": 188,
            "spawns": 632,
            "damage": 20143,
            "uses": 1831,
            "multiKills": 30,
            "vehiclesDestroyedWith": 26,
            "kpm": 1.3,
            "dpm": 139.48,
            "secondsPlayed": 8665
        },
        {
            "type": "Grenade",
            "gadgetName": "Spring Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GAD_ICO_Throwable_Impact-c22cff04.png",
            "id": "open_throw_gren_bb",
            "kills": 5,
            "spawns": 66,
            "damage": 875,
            "uses": 53,
            "multiKills": 3,
            "vehiclesDestroyedWith": 0,
            "kpm": 3.16,
            "dpm": 552.63,
            "secondsPlayed": 95
        },
        {
            "type": "Equipment",
            "gadgetName": "EOD Bot - BF3",
            "image": "https://cdn.gametools.network/gadgets/bf2042/ALX__EODBot-38a9dbbe.png",
            "id": "open_equip_eod",
            "kills": 5,
            "spawns": 21,
            "damage": 933,
            "uses": 9,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.39,
            "dpm": 72.23,
            "secondsPlayed": 775
        },
        {
            "type": "Universal",
            "gadgetName": "Call In Tablet",
            "image": "",
            "id": "uni_callin",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 740
        },
        {
            "type": "Equipment",
            "gadgetName": "Intel Scanner",
            "image": "",
            "id": "open_equip_intscan",
            "kills": 0,
            "spawns": 2,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 50
        },
        {
            "type": "Equipment",
            "gadgetName": "AT-Mine",
            "image": "",
            "id": "open_equip_atmine",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        },
        {
            "type": "Throwable",
            "gadgetName": "Stick Grenade",
            "image": "https://cdn.gametools.network/gadgets/bf2042/GRA__StickGrenade_001-65531a5f.png",
            "id": "",
            "kills": 0,
            "spawns": 0,
            "damage": 0,
            "uses": 0,
            "multiKills": 0,
            "vehiclesDestroyedWith": 0,
            "kpm": 0.0,
            "dpm": 0.0,
            "secondsPlayed": 0
        }
    ],
    "bestClass": "Falck",
    "humanPrecentage": "81.35%",
    "kills": 16953,
    "deaths": 9656,
    "wins": 529,
    "loses": 534,
    "killsPerMinute": 0.78,
    "damagePerMinute": 103.52,
    "killsPerMatch": 15.93,
    "damagePerMatch": 2103.84,
    "headShots": 2992,
    "winPercent": "49.76%",
    "headshots": "17.65%",
    "killDeath": 1.76,
    "infantryKillDeath": 1.43,
    "damage": 2238490,
    "timePlayed": "15 days, 0:24:35",
    "accuracy": "12.13%",
    "revives": 3284,
    "heals": 63481,
    "resupplies": 25900,
    "repairs": 10818,
    "squadmateRevive": 1488,
    "squadmateRespawn": 5,
    "thrownThrowables": 3692,
    "inRound": {
        "revives": 43,
        "resupplies": 14685,
        "squadmateRevive": 3,
        "spotAssists": 84,
        "thrownThrowables": 10,
        "playerTakeDowns": 0,
        "gadgetsDestoyed": 31
    },
    "gadgetsDestoyed": 2052,
    "callIns": 750,
    "playerTakeDowns": 46,
    "matchesPlayed": 1064,
    "secondsPlayed": 1297475,
    "dividedSecondsPlayed": {
        "flying": 65880,
        "driving": 105920
    },
    "bestSquad": 180,
    "teammatesSupported": 4518,
    "saviorKills": 287,
    "shotsFired": 903766,
    "shotsHit": 109619,
    "killAssists": 13019,
    "vehiclesDestroyed": 669,
    "enemiesSpotted": 10436,
    "mvp": 60,
    "dividedKills": {
        "ads": 9924,
        "grenades": 223,
        "hipfire": 2597,
        "longDistance": 859000,
        "melee": 151,
        "multiKills": 3056,
        "parachute": 42,
        "passenger": 1353,
        "vehicle": 2605,
        "roadkills": 117,
        "ai": 3161,
        "human": 13792,
        "turret": 6,
        "ranger": 7,
        "weapons": {
            "BAR": 572,
            "SMG": 795,
            "LMG": 1477,
            "DMR": 1218,
            "Sidearm": 573,
            "Crossbows": 47,
            "Shotguns": 532,
            "Assault Rifles": 532
        },
        "inRound": {
            "total": 165,
            "turret": 0,
            "killsAndAssists": 420,
            "drone": 1,
            "grenade": 7,
            "headshots": 64,
            "hipfire": 4,
            "longDistance": 1080,
            "melee": 10,
            "multiKills": 1,
            "parachute": 1,
            "passenger": 34,
            "weapons": {
                "Sidearm": 62,
                "BAR": 23,
                "SMG": 63,
                "AR": 57,
                "DMR": 100
            }
        }
    },
    "devidedDamage": {
        "explosion": 568396,
        "passenger": 224634,
        "vehicleDriver": 227538,
        "toVehicle": 93482,
        "inRound": {
            "passenger": 3868,
            "explosion": 4314,
            "toVehicle": 1296,
            "asVehicle": 4643
        }
    },
    "devidedAssists": {
        "driver": 1105,
        "passenger": 1909,
        "spot": 3631,
        "pilot": 401,
        "ai": 1075,
        "inRound": {
            "total": 56,
            "passenger": 3,
            "pilot": 2
        }
    },
    "distanceTraveled": {
        "foot": 1916689,
        "grapple": 51525,
        "passenger": 2519594,
        "vehicle": 5960010
    },
    "sector": {
        "captured": 13770,
        "defended": 3777
    },
    "objective": {
        "time": {
            "total": 158165,
            "attacked": 38540,
            "defended": 97675
        },
        "armed": 57,
        "captured": 6357,
        "neutralized": 5625,
        "defused": 24,
        "destroyed": 6,
        "inRound": {
            "time": 0,
            "armed": 46,
            "captured": 8,
            "neutralized": 237,
            "defused": 2,
            "destroyed": 2
        }
    },
    "XP": [
        {
            "total": 26172672,
            "performance": 10763989,
            "ribbons": {
                "total": 6340100,
                "squad": 1518400,
                "combat": 1844900,
                "intel": 760200,
                "objective": 1492500,
                "support": 724100
            }
        }
    ],
    "seasons": {
        "1": {
            "kills": 70,
            "deaths": 103,
            "wins": 7,
            "loses": 2,
            "roundsPlayed": 9,
            "heals": 19,
            "revives": 35,
            "resupplies": 111,
            "assists": 75,
            "headshots": 16,
            "enemiesSpotted": 13,
            "vehiclesDestroyed": 10,
            "ribbons": 40,
            "objective": {
                "armed": 0,
                "captured": 2,
                "defused": 0,
                "destroyed": 0,
                "neutralized": 1
            },
            "missionsCompleted": 11,
            "intelExtracted": 0,
            "timesExtracted": 0,
            "vehiclesHPRepaired": 0
        }
    }
}


async def query_data(player, platform, query_type):
    url = f"https://api.gametools.network/bf2042/stats/?raw=false&format_values=true&name={player}&platform={platform}"
    headers = {
        'accept': 'application/json'
    }
    mes = (False, "没有获取到数据呢")
    retry_options = ExponentialRetry(attempts=2, exceptions=(aiohttp.ClientError,))
    async with RetryClient(retry_options=retry_options) as session:
        try:
            async with session.get(url, headers=headers, timeout=15) as response:
                rest = await response.text()
                # rest = str_filter(rest)
                if response.status == 200:
                    result = json.loads(rest)
                    img = await builder(result, platform, query_type)
                    mes = (True, img)
                else:
                    mes = (False, "请求错误")
        except asyncio.TimeoutError as e:
            if e:
                mes = (False, f"请求超时：{e}")
            mes = (False, f"请求超时：玩家数据请求超时")
        except aiohttp.ClientError as e:
            if e:
                mes = (False, f"请求异常：{e}")
            mes = (False, f"请求异常：玩家数据请求异常")
    return mes


obj_filter = {
    "AH-64GX Apache Warchief": "Apache Warchief",
    "GOL Sniper Magnum": "GOL Magnum",
    "AH-6J Little Bird": "Little Bird",
    "Sd. Kfz 251 Halftrack": "251 Halftrack",
    "9K22 Tunguska-M": "Tunguska-M"
}


async def get_img(query_type):
    platform = 'pc'
    img = await builder(data, platform, query_type)
    mes = (True, img)
    return mes


def str_filter(obj_str):
    obj_str = str(obj_str)
    # 遍历字典，替换字符串中的键为对应的值
    for key in obj_filter:
        obj_str = obj_str.replace(key, obj_filter[key])
    return obj_str
